@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -20,6 +20,7 @@ package org.apache.catalina.authenticator;
 
 
 import java.io.IOException;
+import java.io.StringReader;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.security.Principal;
@@ -33,11 +34,13 @@ import javax.servlet.http.HttpServletResponse;
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.Realm;
 import org.apache.catalina.connector.Request;
-import org.apache.catalina.connector.Response;
 import org.apache.catalina.deploy.LoginConfig;
-import org.apache.catalina.util.MD5Encoder;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.buf.B2CConverter;
+import org.apache.tomcat.util.http.parser.HttpParser;
+import org.apache.tomcat.util.security.ConcurrentMessageDigest;
+import org.apache.tomcat.util.security.MD5Encoder;
 
 
 
@@ -47,19 +50,21 @@ import org.apache.juli.logging.LogFactory;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- *
  */
 
 public class DigestAuthenticator extends AuthenticatorBase {
 
-    private static Log log = LogFactory.getLog(DigestAuthenticator.class);
+    private static final Log log = LogFactory.getLog(DigestAuthenticator.class);
 
 
     // -------------------------------------------------------------- Constants
 
     /**
      * The MD5 helper object for this class.
+     *
+     * @deprecated  Unused - will be removed in Tomcat 8.0.x
      */
+    @Deprecated
     protected static final MD5Encoder md5Encoder = new MD5Encoder();
 
 
@@ -85,8 +90,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
             if (md5Helper == null)
                 md5Helper = MessageDigest.getInstance("MD5");
         } catch (NoSuchAlgorithmException e) {
-            e.printStackTrace();
-            throw new IllegalStateException();
+            throw new IllegalStateException(e);
         }
     }
 
@@ -96,8 +100,10 @@ public class DigestAuthenticator extends AuthenticatorBase {
 
     /**
      * MD5 message digest provider.
+     * @deprecated  Unused - will be removed in Tomcat 8.0.x onwards
      */
-    protected static MessageDigest md5Helper;
+    @Deprecated
+    protected static volatile MessageDigest md5Helper;
 
 
     /**
@@ -106,6 +112,14 @@ public class DigestAuthenticator extends AuthenticatorBase {
     protected Map<String,NonceInfo> nonces;
 
 
+    /**
+     * The last timestamp used to generate a nonce. Each nonce should get a
+     * unique timestamp.
+     */
+    protected long lastTimestamp = 0;
+    protected final Object lastTimestampLock = new Object();
+
+
     /**
      * Maximum number of server nonces to keep in the cache. If not specified,
      * the default value of 1000 is used.
@@ -113,6 +127,12 @@ public class DigestAuthenticator extends AuthenticatorBase {
     protected int nonceCacheSize = 1000;
 
 
+    /**
+     * The window size to use to track seen nonce count values for a given
+     * nonce. If not specified, the default of 100 is used.
+     */
+    protected int nonceCountWindowSize = 100;
+
     /**
      * Private key.
      */
@@ -151,6 +171,16 @@ public class DigestAuthenticator extends AuthenticatorBase {
     }
 
 
+    public int getNonceCountWindowSize() {
+        return nonceCountWindowSize;
+    }
+
+
+    public void setNonceCountWindowSize(int nonceCountWindowSize) {
+        this.nonceCountWindowSize = nonceCountWindowSize;
+    }
+
+
     public int getNonceCacheSize() {
         return nonceCacheSize;
     }
@@ -218,50 +248,24 @@ public class DigestAuthenticator extends AuthenticatorBase {
      */
     @Override
     public boolean authenticate(Request request,
-                                Response response,
+                                HttpServletResponse response,
                                 LoginConfig config)
         throws IOException {
 
-        // Have we already authenticated someone?
-        Principal principal = request.getUserPrincipal();
-        //String ssoId = (String) request.getNote(Constants.REQ_SSOID_NOTE);
-        if (principal != null) {
-            if (log.isDebugEnabled())
-                log.debug("Already authenticated '" + principal.getName() + "'");
-            // Associate the session with any existing SSO session in order
-            // to get coordinated session invalidation at logout
-            String ssoId = (String) request.getNote(Constants.REQ_SSOID_NOTE);
-            if (ssoId != null)
-                associate(ssoId, request.getSessionInternal(true));
-            return (true);
-        }
-
         // NOTE: We don't try to reauthenticate using any existing SSO session,
         // because that will only work if the original authentication was
         // BASIC or FORM, which are less secure than the DIGEST auth-type
         // specified for this webapp
         //
-        // Uncomment below to allow previous FORM or BASIC authentications
+        // Change to true below to allow previous FORM or BASIC authentications
         // to authenticate users for this webapp
         // TODO make this a configurable attribute (in SingleSignOn??)
-        /*
-        // Is there an SSO session against which we can try to reauthenticate?
-        if (ssoId != null) {
-            if (log.isDebugEnabled())
-                log.debug("SSO Id " + ssoId + " set; attempting " +
-                          "reauthentication");
-            // Try to reauthenticate using data cached by SSO.  If this fails,
-            // either the original SSO logon was of DIGEST or SSL (which
-            // we can't reauthenticate ourselves because there is no
-            // cached username and password), or the realm denied
-            // the user's reauthentication for some reason.
-            // In either case we have to prompt the user for a logon
-            if (reauthenticateFromSSO(ssoId, request))
-                return true;
+        if (checkForCachedAuthentication(request, response, false)) {
+            return true;
         }
-        */
 
         // Validate any credentials already included with this request
+        Principal principal = null;
         String authorization = request.getHeader("authorization");
         DigestInfo digestInfo = new DigestInfo(getOpaque(), getNonceValidity(),
                 getKey(), nonces, isValidateUri());
@@ -270,7 +274,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
                 if (digestInfo.validate(request, config)) {
                     principal = digestInfo.authenticate(context.getRealm());
                 }
-            
+
                 if (principal != null && !digestInfo.isNonceStale()) {
                     register(request, response, principal,
                             HttpServletRequest.DIGEST_AUTH,
@@ -290,7 +294,12 @@ public class DigestAuthenticator extends AuthenticatorBase {
                 principal != null && digestInfo.isNonceStale());
         response.sendError(HttpServletResponse.SC_UNAUTHORIZED);
         return false;
+    }
 
+
+    @Override
+    protected String getAuthMethod() {
+        return HttpServletRequest.DIGEST_AUTH;
     }
 
 
@@ -371,18 +380,22 @@ public class DigestAuthenticator extends AuthenticatorBase {
 
         long currentTime = System.currentTimeMillis();
 
-        
+        synchronized (lastTimestampLock) {
+            if (currentTime > lastTimestamp) {
+                lastTimestamp = currentTime;
+            } else {
+                currentTime = ++lastTimestamp;
+            }
+        }
+
         String ipTimeKey =
             request.getRemoteAddr() + ":" + currentTime + ":" + getKey();
 
-        byte[] buffer;
-        synchronized (md5Helper) {
-            buffer = md5Helper.digest(ipTimeKey.getBytes());
-        }
+        byte[] buffer = ConcurrentMessageDigest.digestMD5(
+                ipTimeKey.getBytes(B2CConverter.ISO_8859_1));
+        String nonce = currentTime + ":" + MD5Encoder.encode(buffer);
 
-        String nonce = currentTime + ":" + md5Encoder.encode(buffer);
-
-        NonceInfo info = new NonceInfo(currentTime, 100);
+        NonceInfo info = new NonceInfo(currentTime, getNonceCountWindowSize());
         synchronized (nonces) {
             nonces.put(nonce, info);
         }
@@ -418,8 +431,8 @@ public class DigestAuthenticator extends AuthenticatorBase {
      *              should be performed
      * @param nonce nonce token
      */
-    protected void setAuthenticateHeader(Request request,
-                                         Response response,
+    protected void setAuthenticateHeader(HttpServletRequest request,
+                                         HttpServletResponse response,
                                          LoginConfig config,
                                          String nonce,
                                          boolean isNonceStale) {
@@ -440,27 +453,27 @@ public class DigestAuthenticator extends AuthenticatorBase {
             getOpaque() + "\"";
         }
 
-        response.setHeader("WWW-Authenticate", authenticateHeader);
+        response.setHeader(AUTH_HEADER_NAME, authenticateHeader);
 
     }
 
 
     // ------------------------------------------------------- Lifecycle Methods
-    
+
     @Override
-    public void start() throws LifecycleException {
-        super.start();
-        
+    protected synchronized void startInternal() throws LifecycleException {
+        super.startInternal();
+
         // Generate a random secret key
         if (getKey() == null) {
-            setKey(generateSessionId());
+            setKey(sessionIdGenerator.generateSessionId());
         }
-        
+
         // Generate the opaque string the same way
         if (getOpaque() == null) {
-            setOpaque(generateSessionId());
+            setOpaque(sessionIdGenerator.generateSessionId());
         }
-        
+
         nonces = new LinkedHashMap<String, DigestAuthenticator.NonceInfo>() {
 
             private static final long serialVersionUID = 1L;
@@ -488,7 +501,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
             }
         };
     }
- 
+
     private static class DigestInfo {
 
         private final String opaque;
@@ -531,48 +544,29 @@ public class DigestAuthenticator extends AuthenticatorBase {
             if (authorization == null) {
                 return false;
             }
-            if (!authorization.startsWith("Digest ")) {
+
+            Map<String,String> directives;
+            try {
+                directives = HttpParser.parseAuthorizationDigest(
+                        new StringReader(authorization));
+            } catch (IOException e) {
                 return false;
             }
-            authorization = authorization.substring(7).trim();
 
-            // Bugzilla 37132: http://issues.apache.org/bugzilla/show_bug.cgi?id=37132
-            String[] tokens = authorization.split(",(?=(?:[^\"]*\"[^\"]*\")+$)");
+            if (directives == null) {
+                return false;
+            }
 
             method = request.getMethod();
-
-            for (int i = 0; i < tokens.length; i++) {
-                String currentToken = tokens[i];
-                if (currentToken.length() == 0)
-                    continue;
-
-                int equalSign = currentToken.indexOf('=');
-                if (equalSign < 0) {
-                    return false;
-                }
-                String currentTokenName =
-                    currentToken.substring(0, equalSign).trim();
-                String currentTokenValue =
-                    currentToken.substring(equalSign + 1).trim();
-                if ("username".equals(currentTokenName))
-                    userName = removeQuotes(currentTokenValue);
-                if ("realm".equals(currentTokenName))
-                    realmName = removeQuotes(currentTokenValue, true);
-                if ("nonce".equals(currentTokenName))
-                    nonce = removeQuotes(currentTokenValue);
-                if ("nc".equals(currentTokenName))
-                    nc = removeQuotes(currentTokenValue);
-                if ("cnonce".equals(currentTokenName))
-                    cnonce = removeQuotes(currentTokenValue);
-                if ("qop".equals(currentTokenName))
-                    qop = removeQuotes(currentTokenValue);
-                if ("uri".equals(currentTokenName))
-                    uri = removeQuotes(currentTokenValue);
-                if ("response".equals(currentTokenName))
-                    response = removeQuotes(currentTokenValue);
-                if ("opaque".equals(currentTokenName))
-                    opaqueReceived = removeQuotes(currentTokenValue);
-            }
+            userName = directives.get("username");
+            realmName = directives.get("realm");
+            nonce = directives.get("nonce");
+            nc = directives.get("nc");
+            cnonce = directives.get("cnonce");
+            qop = directives.get("qop");
+            uri = directives.get("uri");
+            response = directives.get("response");
+            opaqueReceived = directives.get("opaque");
 
             return true;
         }
@@ -621,7 +615,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
             if (!lcRealm.equals(realmName)) {
                 return false;
             }
-            
+
             // Validate the opaque string
             if (!opaque.equals(opaqueReceived)) {
                 return false;
@@ -648,11 +642,9 @@ public class DigestAuthenticator extends AuthenticatorBase {
             }
             String serverIpTimeKey =
                 request.getRemoteAddr() + ":" + nonceTime + ":" + key;
-            byte[] buffer = null;
-            synchronized (md5Helper) {
-                buffer = md5Helper.digest(serverIpTimeKey.getBytes());
-            }
-            String md5ServerIpTimeKey = md5Encoder.encode(buffer);
+            byte[] buffer = ConcurrentMessageDigest.digestMD5(
+                    serverIpTimeKey.getBytes(B2CConverter.ISO_8859_1));
+            String md5ServerIpTimeKey = MD5Encoder.encode(buffer);
             if (!md5ServerIpTimeKey.equals(md5clientIpTimeKey)) {
                 return false;
             }
@@ -709,11 +701,9 @@ public class DigestAuthenticator extends AuthenticatorBase {
             // MD5(Method + ":" + uri)
             String a2 = method + ":" + uri;
 
-            byte[] buffer;
-            synchronized (md5Helper) {
-                buffer = md5Helper.digest(a2.getBytes());
-            }
-            String md5a2 = md5Encoder.encode(buffer);
+            byte[] buffer = ConcurrentMessageDigest.digestMD5(
+                    a2.getBytes(B2CConverter.ISO_8859_1));
+            String md5a2 = MD5Encoder.encode(buffer);
 
             return realm.authenticate(userName, response, nonce, nc, cnonce,
                     qop, realmName, md5a2);
@@ -722,17 +712,17 @@ public class DigestAuthenticator extends AuthenticatorBase {
     }
 
     private static class NonceInfo {
-        private volatile long timestamp;
-        private volatile boolean seen[];
-        private volatile int offset;
-        private volatile int count = 0;
+        private final long timestamp;
+        private final boolean seen[];
+        private final int offset;
+        private int count = 0;
 
         public NonceInfo(long currentTime, int seenWindowSize) {
             this.timestamp = currentTime;
             seen = new boolean[seenWindowSize];
             offset = seenWindowSize / 2;
         }
-        
+
         public synchronized boolean nonceCountValid(long nonceCount) {
             if ((count - offset) >= nonceCount ||
                     (nonceCount > count - offset + seen.length)) {
@@ -748,7 +738,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
                 return true;
             }
         }
-        
+
         public long getTimestamp() {
             return timestamp;
         }
