@@ -51,6 +51,7 @@ import org.apache.catalina.util.RequestUtil;
 import org.apache.catalina.util.SessionConfig;
 import org.apache.tomcat.util.buf.CharChunk;
 import org.apache.tomcat.util.buf.UEncoder;
+import org.apache.tomcat.util.buf.UEncoder.SafeCharsSet;
 import org.apache.tomcat.util.http.FastHttpDateFormat;
 import org.apache.tomcat.util.http.MimeHeaders;
 import org.apache.tomcat.util.http.ServerCookie;
@@ -91,7 +92,6 @@ public class Response
     }
 
     public Response() {
-        urlEncoder.addSafeCharacter('/');
     }
 
 
@@ -282,7 +282,7 @@ public class Response
     /**
      * URL encoder.
      */
-    protected UEncoder urlEncoder = new UEncoder();
+    protected final UEncoder urlEncoder = new UEncoder(SafeCharsSet.WITH_SLASH);
 
 
     /**
@@ -361,7 +361,7 @@ public class Response
                 // Ignore - the client has probably closed the connection
             }
         }
-        return coyoteResponse.getBytesWritten(flush);
+        return getCoyoteResponse().getBytesWritten(flush);
     }
 
     /**
@@ -558,7 +558,7 @@ public class Response
      * Return the content length that was set or calculated for this Response.
      */
     public int getContentLength() {
-        return coyoteResponse.getContentLength();
+        return getCoyoteResponse().getContentLength();
     }
 
 
@@ -568,7 +568,7 @@ public class Response
      */
     @Override
     public String getContentType() {
-        return coyoteResponse.getContentType();
+        return getCoyoteResponse().getContentType();
     }
 
 
@@ -625,7 +625,7 @@ public class Response
      */
     @Override
     public String getCharacterEncoding() {
-        return (coyoteResponse.getCharacterEncoding());
+        return (getCoyoteResponse().getCharacterEncoding());
     }
 
 
@@ -659,7 +659,7 @@ public class Response
      */
     @Override
     public Locale getLocale() {
-        return (coyoteResponse.getLocale());
+        return (getCoyoteResponse().getLocale());
     }
 
 
@@ -709,7 +709,7 @@ public class Response
      */
     @Override
     public boolean isCommitted() {
-        return coyoteResponse.isCommitted();
+        return getCoyoteResponse().isCommitted();
     }
 
 
@@ -726,7 +726,7 @@ public class Response
             return;
         }
 
-        coyoteResponse.reset();
+        getCoyoteResponse().reset();
         outputBuffer.reset();
         usingOutputStream = false;
         usingWriter = false;
@@ -813,7 +813,7 @@ public class Response
             return;
         }
 
-        coyoteResponse.setContentLength(length);
+        getCoyoteResponse().setContentLength(length);
 
     }
 
@@ -836,7 +836,7 @@ public class Response
         }
 
         if (type == null) {
-            coyoteResponse.setContentType(null);
+            getCoyoteResponse().setContentType(null);
             return;
         }
 
@@ -844,16 +844,16 @@ public class Response
         if (m == null) {
             // Invalid - Assume no charset and just pass through whatever
             // the user provided.
-            coyoteResponse.setContentTypeNoCharset(type);
+            getCoyoteResponse().setContentTypeNoCharset(type);
             return;
         }
 
-        coyoteResponse.setContentTypeNoCharset(m[0]);
+        getCoyoteResponse().setContentTypeNoCharset(m[0]);
 
         if (m[1] != null) {
             // Ignore charset if getWriter() has already been called
             if (!usingWriter) {
-                coyoteResponse.setCharacterEncoding(m[1]);
+                getCoyoteResponse().setCharacterEncoding(m[1]);
                 isCharacterEncodingSet = true;
             }
         }
@@ -885,7 +885,7 @@ public class Response
             return;
         }
 
-        coyoteResponse.setCharacterEncoding(charset);
+        getCoyoteResponse().setCharacterEncoding(charset);
         isCharacterEncodingSet = true;
     }
 
@@ -908,7 +908,7 @@ public class Response
             return;
         }
 
-        coyoteResponse.setLocale(locale);
+        getCoyoteResponse().setLocale(locale);
 
         // Ignore any call made after the getWriter has been invoked.
         // The default should be used
@@ -922,7 +922,7 @@ public class Response
 
         String charset = getContext().getCharset(locale);
         if (charset != null) {
-            coyoteResponse.setCharacterEncoding(charset);
+            getCoyoteResponse().setCharacterEncoding(charset);
         }
     }
 
@@ -932,14 +932,14 @@ public class Response
 
     @Override
     public String getHeader(String name) {
-        return coyoteResponse.getMimeHeaders().getHeader(name);
+        return getCoyoteResponse().getMimeHeaders().getHeader(name);
     }
 
 
     @Override
     public Collection<String> getHeaderNames() {
 
-        MimeHeaders headers = coyoteResponse.getMimeHeaders();
+        MimeHeaders headers = getCoyoteResponse().getMimeHeaders();
         int n = headers.size();
         List<String> result = new ArrayList<String>(n);
         for (int i = 0; i < n; i++) {
@@ -954,7 +954,7 @@ public class Response
     public Collection<String> getHeaders(String name) {
 
         Enumeration<String> enumeration =
-            coyoteResponse.getMimeHeaders().values(name);
+                getCoyoteResponse().getMimeHeaders().values(name);
         Vector<String> result = new Vector<String>();
         while (enumeration.hasMoreElements()) {
             result.addElement(enumeration.nextElement());
@@ -968,13 +968,13 @@ public class Response
      * for this Response.
      */
     public String getMessage() {
-        return coyoteResponse.getMessage();
+        return getCoyoteResponse().getMessage();
     }
 
 
     @Override
     public int getStatus() {
-        return coyoteResponse.getStatus();
+        return getCoyoteResponse().getStatus();
     }
 
 
@@ -1032,7 +1032,7 @@ public class Response
         final String startsWith = name + "=";
         final StringBuffer sb = generateCookieString(cookie);
         boolean set = false;
-        MimeHeaders headers = coyoteResponse.getMimeHeaders();
+        MimeHeaders headers = getCoyoteResponse().getMimeHeaders();
         int n = headers.size();
         for (int i = 0; i < n; i++) {
             if (headers.getName(i).toString().equals(headername)) {
@@ -1138,7 +1138,7 @@ public class Response
             return;
         }
 
-        coyoteResponse.addHeader(name, value);
+        getCoyoteResponse().addHeader(name, value);
     }
 
 
@@ -1200,15 +1200,15 @@ public class Response
         if(cc=='C' || cc=='c') {
             if(name.equalsIgnoreCase("Content-Type")) {
                 // Will return null if this has not been set
-                return (coyoteResponse.getContentType() != null);
+                return (getCoyoteResponse().getContentType() != null);
             }
             if(name.equalsIgnoreCase("Content-Length")) {
                 // -1 means not known and is not sent to client
-                return (coyoteResponse.getContentLengthLong() != -1);
+                return (getCoyoteResponse().getContentLengthLong() != -1);
             }
         }
 
-        return coyoteResponse.containsHeader(name);
+        return getCoyoteResponse().containsHeader(name);
     }
 
 
@@ -1311,8 +1311,7 @@ public class Response
             return;
         }
 
-        coyoteResponse.acknowledge();
-
+        getCoyoteResponse().acknowledge();
     }
 
 
@@ -1357,8 +1356,8 @@ public class Response
 
         setError();
 
-        coyoteResponse.setStatus(status);
-        coyoteResponse.setMessage(message);
+        getCoyoteResponse().setStatus(status);
+        getCoyoteResponse().setMessage(message);
 
         // Clear any data content that has been buffered
         resetBuffer();
@@ -1473,7 +1472,7 @@ public class Response
             return;
         }
 
-        coyoteResponse.setHeader(name, value);
+        getCoyoteResponse().setHeader(name, value);
     }
 
 
@@ -1538,8 +1537,8 @@ public class Response
             return;
         }
 
-        coyoteResponse.setStatus(status);
-        coyoteResponse.setMessage(message);
+        getCoyoteResponse().setStatus(status);
+        getCoyoteResponse().setMessage(message);
 
     }
 
