@@ -17,7 +17,6 @@
 
 package org.apache.catalina.ha.session;
 
-import java.beans.PropertyChangeEvent;
 import java.io.BufferedOutputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
@@ -27,26 +26,19 @@ import java.util.ArrayList;
 import java.util.Date;
 import java.util.Iterator;
 
-import org.apache.catalina.Cluster;
-import org.apache.catalina.Container;
 import org.apache.catalina.Context;
 import org.apache.catalina.Engine;
 import org.apache.catalina.Host;
 import org.apache.catalina.LifecycleException;
-import org.apache.catalina.LifecycleListener;
+import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Session;
-import org.apache.catalina.Valve;
-import org.apache.catalina.core.StandardContext;
-import org.apache.catalina.ha.CatalinaCluster;
+import org.apache.catalina.ha.ClusterManager;
 import org.apache.catalina.ha.ClusterMessage;
-import org.apache.catalina.ha.tcp.ReplicationValve;
 import org.apache.catalina.session.ManagerBase;
-import org.apache.catalina.session.TooManyActiveSessionsException;
 import org.apache.catalina.tribes.Member;
 import org.apache.catalina.tribes.io.ReplicationStream;
-import org.apache.catalina.util.LifecycleSupport;
-import org.apache.catalina.util.StringManager;
-import org.apache.catalina.ha.ClusterManager;
+import org.apache.tomcat.util.ExceptionUtils;
+import org.apache.tomcat.util.res.StringManager;
 
 /**
  * The DeltaManager manages replicated sessions by only replicating the deltas
@@ -64,18 +56,16 @@ import org.apache.catalina.ha.ClusterManager;
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
  * @author Peter Rossbach
- *
  */
-
 public class DeltaManager extends ClusterManagerBase{
 
     // ---------------------------------------------------- Security Classes
-    public static org.apache.juli.logging.Log log = org.apache.juli.logging.LogFactory.getLog(DeltaManager.class);
+    public final org.apache.juli.logging.Log log = org.apache.juli.logging.LogFactory.getLog(DeltaManager.class);
 
     /**
      * The string manager for this package.
      */
-    protected static StringManager sm = StringManager.getManager(Constants.Package);
+    protected static final StringManager sm = StringManager.getManager(Constants.Package);
 
     // ----------------------------------------------------- Instance Variables
 
@@ -84,58 +74,34 @@ public class DeltaManager extends ClusterManagerBase{
      */
     private static final String info = "DeltaManager/2.1";
 
-    /**
-     * Has this component been started yet?
-     */
-    private boolean started = false;
-
     /**
      * The descriptive name of this Manager implementation (for logging).
      */
     protected static String managerName = "DeltaManager";
     protected String name = null;
-    protected boolean defaultMode = false;
-    private CatalinaCluster cluster = null;
-
-    /**
-     * cached replication valve cluster container!
-     */
-    private ReplicationValve replicationValve = null ;
     
-    /**
-     * The lifecycle event support for this component.
-     */
-    protected LifecycleSupport lifecycle = new LifecycleSupport(this);
-
-    /**
-     * The maximum number of active Sessions allowed, or -1 for no limit.
-     */
-    private int maxActiveSessions = -1;
     private boolean expireSessionsOnShutdown = false;
-    private boolean notifyListenersOnReplication = true;
     private boolean notifySessionListenersOnReplication = true;
     private boolean notifyContainerListenersOnReplication  = true;
     private volatile boolean stateTransfered = false ;
     private volatile boolean noContextManagerReceived = false ;
     private int stateTransferTimeout = 60;
     private boolean sendAllSessions = true;
-    private boolean sendClusterDomainOnly = true ;
     private int sendAllSessionsSize = 1000 ;
     
     /**
      * wait time between send session block (default 2 sec) 
      */
     private int sendAllSessionsWaitTime = 2 * 1000 ; 
-    private ArrayList receivedMessageQueue = new ArrayList() ;
+    private ArrayList<SessionMessage> receivedMessageQueue =
+        new ArrayList<SessionMessage>() ;
     private boolean receiverQueue = false ;
     private boolean stateTimestampDrop = true ;
     private long stateTransferCreateSendTime; 
     
     // ------------------------------------------------------------------ stats attributes
     
-    int rejectedSessions = 0;
     private long sessionReplaceCounter = 0 ;
-    long processingTime = 0;
     private long counterReceive_EVT_GET_ALL_SESSIONS = 0 ;
     private long counterReceive_EVT_ALL_SESSION_DATA = 0 ;
     private long counterReceive_EVT_SESSION_CREATED = 0 ;
@@ -154,6 +120,7 @@ public class DeltaManager extends ClusterManagerBase{
     private int counterSend_EVT_ALL_SESSION_TRANSFERCOMPLETE = 0 ;
     private long counterSend_EVT_CHANGE_SESSION_ID = 0;
     private int counterNoStateTransfered = 0 ;
+    
 
     // ------------------------------------------------------------- Constructor
     public DeltaManager() {
@@ -167,10 +134,12 @@ public class DeltaManager extends ClusterManagerBase{
      * corresponding version number, in the format
      * <code>&lt;description&gt;/&lt;version&gt;</code>.
      */
+    @Override
     public String getInfo() {
         return info;
     }
 
+    @Override
     public void setName(String name) {
         this.name = name;
     }
@@ -178,6 +147,7 @@ public class DeltaManager extends ClusterManagerBase{
     /**
      * Return the descriptive short name of this Manager implementation.
      */
+    @Override
     public String getName() {
         return name;
     }
@@ -305,6 +275,7 @@ public class DeltaManager extends ClusterManagerBase{
     /**
      * @return Returns the processingTime.
      */
+    @Override
     public long getProcessingTime() {
         return processingTime;
     }
@@ -316,19 +287,6 @@ public class DeltaManager extends ClusterManagerBase{
         return sessionReplaceCounter;
     }
     
-    /**
-     * Number of session creations that failed due to maxActiveSessions
-     * 
-     * @return The count
-     */
-    public int getRejectedSessions() {
-        return rejectedSessions;
-    }
-
-    public void setRejectedSessions(int rejectedSessions) {
-        this.rejectedSessions = rejectedSessions;
-    }
-
     /**
      * @return Returns the counterNoStateTransfered.
      */
@@ -391,20 +349,6 @@ public class DeltaManager extends ClusterManagerBase{
         this.sendAllSessionsWaitTime = sendAllSessionsWaitTime;
     }
     
-    /**
-     * @return Returns the sendClusterDomainOnly.
-     */
-    public boolean doDomainReplication() {
-        return sendClusterDomainOnly;
-    }
-    
-    /**
-     * @param sendClusterDomainOnly The sendClusterDomainOnly to set.
-     */
-    public void setDomainReplication(boolean sendClusterDomainOnly) {
-        this.sendClusterDomainOnly = sendClusterDomainOnly;
-    }
-
     /**
      * @return Returns the stateTimestampDrop.
      */
@@ -419,25 +363,6 @@ public class DeltaManager extends ClusterManagerBase{
         this.stateTimestampDrop = isTimestampDrop;
     }
     
-    /**
-     * Return the maximum number of active Sessions allowed, or -1 for no limit.
-     */
-    public int getMaxActiveSessions() {
-        return (this.maxActiveSessions);
-    }
-
-    /**
-     * Set the maximum number of actives Sessions allowed, or -1 for no limit.
-     * 
-     * @param max
-     *            The new maximum number of sessions
-     */
-    public void setMaxActiveSessions(int max) {
-        int oldMaxActiveSessions = this.maxActiveSessions;
-        this.maxActiveSessions = max;
-        support.firePropertyChange("maxActiveSessions", new Integer(oldMaxActiveSessions), new Integer(this.maxActiveSessions));
-    }
-    
     /**
      * 
      * @return Returns the sendAllSessions.
@@ -490,14 +415,6 @@ public class DeltaManager extends ClusterManagerBase{
         this.expireSessionsOnShutdown = expireSessionsOnShutdown;
     }
     
-    public boolean isNotifyListenersOnReplication() {
-        return notifyListenersOnReplication;
-    }
-
-    public void setNotifyListenersOnReplication(boolean notifyListenersOnReplication) {
-        this.notifyListenersOnReplication = notifyListenersOnReplication;
-    }
-
     public boolean isNotifyContainerListenersOnReplication() {
         return notifyContainerListenersOnReplication;
     }
@@ -507,51 +424,6 @@ public class DeltaManager extends ClusterManagerBase{
         this.notifyContainerListenersOnReplication = notifyContainerListenersOnReplication;
     }
     
-    /**
-     * @return Returns the defaultMode.
-     */
-    public boolean isDefaultMode() {
-        return defaultMode;
-    }
-    /**
-     * @param defaultMode The defaultMode to set.
-     */
-    public void setDefaultMode(boolean defaultMode) {
-        this.defaultMode = defaultMode;
-    }
-    
-    public CatalinaCluster getCluster() {
-        return cluster;
-    }
-
-    public void setCluster(CatalinaCluster cluster) {
-        this.cluster = cluster;
-    }
-
-    /**
-     * Set the Container with which this Manager has been associated. If it is a
-     * Context (the usual case), listen for changes to the session timeout
-     * property.
-     * 
-     * @param container
-     *            The associated Container
-     */
-    public void setContainer(Container container) {
-        // De-register from the old Container (if any)
-        if ((this.container != null) && (this.container instanceof Context))
-            ((Context) this.container).removePropertyChangeListener(this);
-
-        // Default processing provided by our superclass
-        super.setContainer(container);
-
-        // Register with the new Container (if any)
-        if ((this.container != null) && (this.container instanceof Context)) {
-            setMaxInactiveInterval(((Context) this.container).getSessionTimeout() * 60);
-            ((Context) this.container).addPropertyChangeListener(this);
-        }
-
-    }
-    
     // --------------------------------------------------------- Public Methods
 
     /**
@@ -573,6 +445,7 @@ public class DeltaManager extends ClusterManagerBase{
      * @exception IllegalStateException
      *                if a new session cannot be instantiated for any reason
      */
+    @Override
     public Session createSession(String sessionId) {
         return createSession(sessionId, true);
     }
@@ -585,20 +458,14 @@ public class DeltaManager extends ClusterManagerBase{
      * @return The session
      */
     public Session createSession(String sessionId, boolean distribute) {
-        if ((maxActiveSessions >= 0) && (sessions.size() >= maxActiveSessions)) {
-            rejectedSessions++;
-            throw new TooManyActiveSessionsException(
-                    sm.getString("deltaManager.createSession.ise"),
-                    maxActiveSessions);
-        }
         DeltaSession session = (DeltaSession) super.createSession(sessionId) ;
         if (distribute) {
             sendCreateSession(session.getId(), session);
         }
         if (log.isDebugEnabled())
-            log.debug(sm.getString("deltaManager.createSession.newSession",session.getId(), new Integer(sessions.size())));
+            log.debug(sm.getString("deltaManager.createSession.newSession",
+                    session.getId(), Integer.valueOf(sessions.size())));
         return (session);
-
     }
 
     /**
@@ -627,10 +494,7 @@ public class DeltaManager extends ClusterManagerBase{
      */
     protected void send(SessionMessage msg) {
         if(cluster != null) {
-            if(doDomainReplication())
-                cluster.sendClusterDomain(msg);
-            else
-                cluster.send(msg);
+            cluster.send(msg);
         }
     }
 
@@ -638,6 +502,7 @@ public class DeltaManager extends ClusterManagerBase{
      * Create DeltaSession
      * @see org.apache.catalina.Manager#createEmptySession()
      */
+    @Override
     public Session createEmptySession() {
         return getNewDeltaSession() ;
     }
@@ -664,7 +529,7 @@ public class DeltaManager extends ClusterManagerBase{
         // original sessionID
         String orgSessionID = session.getId();
         super.changeSessionId(session);
-        if (notify) {
+        if (notify && cluster.getMembers().length > 0) {
             // changed sessionID
             String newSessionID = session.getId();
             try {
@@ -750,7 +615,7 @@ public class DeltaManager extends ClusterManagerBase{
 
     /**
      * Load sessions from other cluster node.
-     * FIXME replace currently sessions with same id without notifcation.
+     * FIXME replace currently sessions with same id without notification.
      * FIXME SSO handling is not really correct with the session replacement!
      * @exception ClassNotFoundException
      *                if a serialized class cannot be found during the reload
@@ -817,7 +682,6 @@ public class DeltaManager extends ClusterManagerBase{
 
     }
 
-    
 
     /**
      * Save any currently active sessions in the appropriate persistence
@@ -836,7 +700,7 @@ public class DeltaManager extends ClusterManagerBase{
         try {
             fos = new ByteArrayOutputStream();
             oos = new ObjectOutputStream(new BufferedOutputStream(fos));
-            oos.writeObject(new Integer(currentSessions.length));
+            oos.writeObject(Integer.valueOf(currentSessions.length));
             for(int i=0 ; i < currentSessions.length;i++) {
                 ((DeltaSession)currentSessions[i]).writeObjectData(oos);                
             }
@@ -850,7 +714,7 @@ public class DeltaManager extends ClusterManagerBase{
                 try {
                     oos.close();
                 } catch (IOException f) {
-                    ;
+                    // Ignore
                 }
                 oos = null;
             }
@@ -859,84 +723,20 @@ public class DeltaManager extends ClusterManagerBase{
         return fos.toByteArray();
     }
 
-    // ------------------------------------------------------ Lifecycle Methods
-
     /**
-     * Add a lifecycle event listener to this component.
-     * 
-     * @param listener
-     *            The listener to add
+     * Start this component and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
+     *
+     * @exception LifecycleException if this component detects a fatal error
+     *  that prevents this component from being used
      */
-    public void addLifecycleListener(LifecycleListener listener) {
-        lifecycle.addLifecycleListener(listener);
-    }
+    @Override
+    protected synchronized void startInternal() throws LifecycleException {
 
-    /**
-     * Get the lifecycle listeners associated with this lifecycle. If this
-     * Lifecycle has no listeners registered, a zero-length array is returned.
-     */
-    public LifecycleListener[] findLifecycleListeners() {
-        return lifecycle.findLifecycleListeners();
-    }
-
-    /**
-     * Remove a lifecycle event listener from this component.
-     * 
-     * @param listener
-     *            The listener to remove
-     */
-    public void removeLifecycleListener(LifecycleListener listener) {
-        lifecycle.removeLifecycleListener(listener);
-    }
-
-    /**
-     * Prepare for the beginning of active use of the public methods of this
-     * component. This method should be called after <code>configure()</code>,
-     * and before any of the public methods of the component are utilized.
-     * 
-     * @exception LifecycleException
-     *                if this component detects a fatal error that prevents this
-     *                component from being used
-     */
-    public void start() throws LifecycleException {
-        if (!initialized) init();
-
-        // Validate and update our current component state
-        if (started) {
-            return;
-        }
-        started = true;
-        lifecycle.fireLifecycleEvent(START_EVENT, null);
-
-        // Force initialization of the random number generator
-        generateSessionId();
+        super.startInternal();
 
         // Load unloaded sessions, if any
         try {
-            //the channel is already running
-            Cluster cluster = getCluster() ;
-            // stop remove cluster binding
-            //wow, how many nested levels of if statements can we have ;)
-            if(cluster == null) {
-                Container context = getContainer();
-                Container host = context.getParent() ;
-                if (host instanceof Host) {
-                    cluster = host.getCluster();
-                    if (cluster instanceof CatalinaCluster) {
-                        setCluster((CatalinaCluster) cluster) ;
-                    } else {
-                        Container engine = host.getParent() ;
-                        if (engine instanceof Engine) {
-                            cluster = engine.getCluster();
-                            if (cluster instanceof CatalinaCluster) {
-                                setCluster((CatalinaCluster) cluster) ;
-                            }
-                        } else {
-                            cluster = null ;
-                        }
-                    }
-                }
-            }
             if (cluster == null) {
                 log.error(sm.getString("deltaManager.noCluster", getName()));
                 return;
@@ -952,15 +752,15 @@ public class DeltaManager extends ClusterManagerBase{
                 }
             }
             if (log.isInfoEnabled()) log.info(sm.getString("deltaManager.startClustering", getName()));
-            //to survice context reloads, as only a stop/start is called, not
-            // createManager
-            cluster.registerManager(this);
 
             getAllClusterSessions();
 
         } catch (Throwable t) {
+            ExceptionUtils.handleThrowable(t);
             log.error(sm.getString("deltaManager.managerLoad"), t);
         }
+        
+        setState(LifecycleState.STARTING);
     }
 
     /**
@@ -975,6 +775,7 @@ public class DeltaManager extends ClusterManagerBase{
                  return;
             }
             SessionMessage msg = new SessionMessageImpl(this.getName(),SessionMessage.EVT_GET_ALL_SESSIONS, null, "GET-ALL","GET-ALL-" + getName());
+            msg.setTimestamp(beforeSendTime);
             // set reference time
             stateTransferCreateSendTime = beforeSendTime ;
             // request session state
@@ -986,13 +787,14 @@ public class DeltaManager extends ClusterManagerBase{
                      receiverQueue = true ;
                 }
                 cluster.send(msg, mbr);
-                if (log.isWarnEnabled()) log.warn(sm.getString("deltaManager.waitForSessionState",getName(), mbr,getStateTransferTimeout()));
+                if (log.isInfoEnabled())
+                    log.info(sm.getString("deltaManager.waitForSessionState",getName(), mbr, Integer.valueOf(getStateTransferTimeout())));
                 // FIXME At sender ack mode this method check only the state transfer and resend is a problem!
                 waitForSendAllSessions(beforeSendTime);
             } finally {
                 synchronized(receivedMessageQueue) {
-                    for (Iterator iter = receivedMessageQueue.iterator(); iter.hasNext();) {
-                        SessionMessage smsg = (SessionMessage) iter.next();
+                    for (Iterator<SessionMessage> iter = receivedMessageQueue.iterator(); iter.hasNext();) {
+                        SessionMessage smsg = iter.next();
                         if (!stateTimestampDrop) {
                             messageReceived(smsg, smsg.getAddress() != null ? (Member) smsg.getAddress() : null);
                         } else {
@@ -1015,33 +817,6 @@ public class DeltaManager extends ClusterManagerBase{
         }
     }
 
-    /**
-     * Register cross context session at replication valve thread local
-     * @param session cross context session
-     */
-    protected void registerSessionAtReplicationValve(DeltaSession session) {
-        if(replicationValve == null) {
-            if(container instanceof StandardContext && ((StandardContext)container).getCrossContext()) {
-                Cluster cluster = getCluster() ;
-                if(cluster != null && cluster instanceof CatalinaCluster) {
-                    Valve[] valves = ((CatalinaCluster)cluster).getValves();
-                    if(valves != null && valves.length > 0) {
-                        for(int i=0; replicationValve == null && i < valves.length ; i++ ){
-                            if(valves[i] instanceof ReplicationValve) replicationValve = (ReplicationValve)valves[i] ;
-                        }//for
-
-                        if(replicationValve == null && log.isDebugEnabled()) {
-                            log.debug("no ReplicationValve found for CrossContext Support");
-                        }//endif 
-                    }//end if
-                }//endif
-            }//end if
-        }//end if
-        if(replicationValve != null) {
-            replicationValve.registerReplicationSession(session);
-        }
-    }
-    
     /**
      * Find the master of the session state
      * @return master member of sessions 
@@ -1072,7 +847,7 @@ public class DeltaManager extends ClusterManagerBase{
                     //
                 }
                 reqNow = System.currentTimeMillis();
-                isTimeout = ((reqNow - reqStart) > (1000 * getStateTransferTimeout()));
+                isTimeout = ((reqNow - reqStart) > (1000L * getStateTransferTimeout()));
             } while ((!getStateTransfered()) && (!isTimeout) && (!isNoContextManagerReceived()));
         } else {
             if(getStateTransferTimeout() == -1) {
@@ -1089,7 +864,7 @@ public class DeltaManager extends ClusterManagerBase{
         if (isTimeout) {
             counterNoStateTransfered++ ;
             log.error(sm.getString("deltaManager.noSessionState",getName(),new Date(beforeSendTime),Long.valueOf(reqNow - beforeSendTime)));
-        } else if (isNoContextManagerReceived()) {
+        }else if (isNoContextManagerReceived()) {
             if (log.isWarnEnabled())
                 log.warn(sm.getString("deltaManager.noContextManager",getName(),new Date(beforeSendTime),Long.valueOf(reqNow - beforeSendTime)));
         } else {
@@ -1099,26 +874,20 @@ public class DeltaManager extends ClusterManagerBase{
     }
 
     /**
-     * Gracefully terminate the active use of the public methods of this
-     * component. This method should be the last one called on a given instance
-     * of this component.
-     * 
-     * @exception LifecycleException
-     *                if this component detects a fatal error that needs to be
-     *                reported
+     * Stop this component and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
+     *
+     * @exception LifecycleException if this component detects a fatal error
+     *  that prevents this component from being used
      */
-    public void stop() throws LifecycleException {
+    @Override
+    protected synchronized void stopInternal() throws LifecycleException {
 
         if (log.isDebugEnabled())
             log.debug(sm.getString("deltaManager.stopped", getName()));
 
-
-        // Validate and update our current component state
-        if (!started)
-            throw new LifecycleException(sm.getString("deltaManager.notStarted"));
-        lifecycle.fireLifecycleEvent(STOP_EVENT, null);
-        started = false;
-
+        setState(LifecycleState.STOPPING);
+        
         // Expire all active sessions
         if (log.isInfoEnabled()) log.info(sm.getString("deltaManager.expireSessions", getName()));
         Session sessions[] = findSessions();
@@ -1128,42 +897,13 @@ public class DeltaManager extends ClusterManagerBase{
                 continue;
             try {
                 session.expire(true, isExpireSessionsOnShutdown());
-            } catch (Throwable ignore) {
-                ;
+            } catch (Throwable t) {
+                ExceptionUtils.handleThrowable(t);
             } 
         }
 
         // Require a new random number generator if we are restarted
-        this.random = null;
-        getCluster().removeManager(this);
-        replicationValve = null;
-        if (initialized) {
-            destroy();
-        }
-    }
-
-    // ----------------------------------------- PropertyChangeListener Methods
-
-    /**
-     * Process property change events from our associated Context.
-     * 
-     * @param event
-     *            The property change event that has occurred
-     */
-    public void propertyChange(PropertyChangeEvent event) {
-
-        // Validate the source of this event
-        if (!(event.getSource() instanceof Context))
-            return;
-        // Process a relevant property change
-        if (event.getPropertyName().equals("sessionTimeout")) {
-            try {
-                setMaxInactiveInterval(((Integer) event.getNewValue()).intValue() * 60);
-            } catch (NumberFormatException e) {
-                log.error(sm.getString("deltaManager.sessionTimeout", event.getNewValue()));
-            }
-        }
-
+        super.stopInternal();
     }
 
     // -------------------------------------------------------- Replication
@@ -1176,6 +916,7 @@ public class DeltaManager extends ClusterManagerBase{
      * @param cmsg -
      *            the message received.
      */
+    @Override
     public void messageDataReceived(ClusterMessage cmsg) {
         if (cmsg != null && cmsg instanceof SessionMessage) {
             SessionMessage msg = (SessionMessage) cmsg;
@@ -1215,32 +956,33 @@ public class DeltaManager extends ClusterManagerBase{
      *            the sessionId that just completed.
      * @return a SessionMessage to be sent,
      */
+    @Override
     public ClusterMessage requestCompleted(String sessionId) {
-        return requestCompleted(sessionId, false);
-    }
+         return requestCompleted(sessionId, false);
+     }
 
-    /**
-     * When the request has been completed, the replication valve will notify
-     * the manager, and the manager will decide whether any replication is
-     * needed or not. If there is a need for replication, the manager will
-     * create a session message and that will be replicated. The cluster
-     * determines where it gets sent.
-     * 
-     * Session expiration also calls this method, but with expires == true.
-     * 
-     * @param sessionId -
-     *            the sessionId that just completed.
-     * @param expires -
-     *            whether this method has been called during session expiration
-     * @return a SessionMessage to be sent,
-     */
-    public ClusterMessage requestCompleted(String sessionId, boolean expires) {
+     /**
+      * When the request has been completed, the replication valve will notify
+      * the manager, and the manager will decide whether any replication is
+      * needed or not. If there is a need for replication, the manager will
+      * create a session message and that will be replicated. The cluster
+      * determines where it gets sent.
+      * 
+      * Session expiration also calls this method, but with expires == true.
+      * 
+      * @param sessionId -
+      *            the sessionId that just completed.
+      * @param expires -
+      *            whether this method has been called during session expiration
+      * @return a SessionMessage to be sent,
+      */
+     public ClusterMessage requestCompleted(String sessionId, boolean expires) {
         DeltaSession session = null;
         try {
             session = (DeltaSession) findSession(sessionId);
             if (session == null) {
                 // A parallel request has called session.invalidate() which has
-                // remove the session from the Manager.
+                // removed the session from the Manager.
                 return null;
             }
             DeltaRequest deltaRequest = session.getDeltaRequest();
@@ -1283,7 +1025,7 @@ public class DeltaManager extends ClusterManagerBase{
             if (!expires && (msg == null)) {
                 long replDelta = System.currentTimeMillis() - session.getLastTimeReplicated();
                 if (session.getMaxInactiveInterval() >=0 && 
-                        replDelta > (session.getMaxInactiveInterval() * 1000)) {
+                        replDelta > (session.getMaxInactiveInterval() * 1000L)) {
                     counterSend_EVT_SESSION_ACCESSED++;
                     msg = new SessionMessageImpl(getName(),
                                                  SessionMessage.EVT_SESSION_ACCESSED, 
@@ -1316,7 +1058,7 @@ public class DeltaManager extends ClusterManagerBase{
      */
     public synchronized void resetStatistics() {
         processingTime = 0 ;
-        expiredSessions = 0 ;
+        expiredSessions.set(0);
         synchronized (sessionCreationTiming) {
             sessionCreationTiming.clear();
             while (sessionCreationTiming.size() <
@@ -1355,16 +1097,6 @@ public class DeltaManager extends ClusterManagerBase{
         
     }
    
-    //  -------------------------------------------------------- persistence handler
-
-    public void load() {
-
-    }
-
-    public void unload() {
-
-    }
-
     //  -------------------------------------------------------- expire
 
     /**
@@ -1374,11 +1106,14 @@ public class DeltaManager extends ClusterManagerBase{
      *            session id
      */
     protected void sessionExpired(String id) {
-        counterSend_EVT_SESSION_EXPIRED++ ;
-        SessionMessage msg = new SessionMessageImpl(getName(),SessionMessage.EVT_SESSION_EXPIRED, null, id, id+ "-EXPIRED-MSG");
-        msg.setTimestamp(System.currentTimeMillis());
-        if (log.isDebugEnabled()) log.debug(sm.getString("deltaManager.createMessage.expire",getName(), id));
-        send(msg);
+        if(cluster.getMembers().length > 0 ) {
+            counterSend_EVT_SESSION_EXPIRED++ ;
+            SessionMessage msg = new SessionMessageImpl(getName(),
+                    SessionMessage.EVT_SESSION_EXPIRED, null, id, id+ "-EXPIRED-MSG");
+            msg.setTimestamp(System.currentTimeMillis());
+            if (log.isDebugEnabled()) log.debug(sm.getString("deltaManager.createMessage.expire",getName(), id));
+            send(msg);
+        }
     }
 
     /**
@@ -1417,28 +1152,13 @@ public class DeltaManager extends ClusterManagerBase{
      * 
      * @return The invalidated sessions array
      */
+    @Override
     public String[] getInvalidatedSessions() {
         return new String[0];
     }
 
     //  -------------------------------------------------------- message receive
 
-    /**
-     * Test that sender and local domain is the same
-     */
-    protected boolean checkSenderDomain(SessionMessage msg,Member sender) {
-        boolean sameDomain= true;
-        if (!sameDomain && log.isWarnEnabled()) {
-                log.warn(sm.getString("deltaManager.receiveMessage.fromWrongDomain",
-                         new Object[] {getName(), 
-                         msg.getEventTypeString(), 
-                         sender,
-                         "",
-                         "" }));
-        }
-        return sameDomain ;
-    }
-
     /**
      * This method is called by the received thread when a SessionMessage has
      * been received from one of the other nodes in the cluster.
@@ -1451,9 +1171,6 @@ public class DeltaManager extends ClusterManagerBase{
      *            requesting node
      */
     protected void messageReceived(SessionMessage msg, Member sender) {
-        if(doDomainReplication() && !checkSenderDomain(msg,sender)) {
-            return;
-        }
         ClassLoader contextLoader = Thread.currentThread().getContextClassLoader();
         try {
             
@@ -1520,7 +1237,7 @@ public class DeltaManager extends ClusterManagerBase{
      */
     protected void handleALL_SESSION_TRANSFERCOMPLETE(SessionMessage msg, Member sender) {
         counterReceive_EVT_ALL_SESSION_TRANSFERCOMPLETE++ ;
-        if (log.isDebugEnabled()) log.debug(sm.getString("deltaManager.receiveMessage.transfercomplete",getName(), sender.getHost(), new Integer(sender.getPort())));
+        if (log.isDebugEnabled()) log.debug(sm.getString("deltaManager.receiveMessage.transfercomplete",getName(), sender.getHost(), Integer.valueOf(sender.getPort())));
         stateTransferCreateSendTime = msg.getTimestamp() ;
         stateTransfered = true ;
     }
@@ -1541,7 +1258,7 @@ public class DeltaManager extends ClusterManagerBase{
             try {
                 session.lock();
                 DeltaRequest dreq = deserializeDeltaRequest(session, delta);
-                dreq.execute(session, notifyListenersOnReplication);
+                dreq.execute(session, isNotifyListenersOnReplication());
                 session.setPrimarySession(false);
             }finally {
                 session.unlock();
@@ -1595,7 +1312,7 @@ public class DeltaManager extends ClusterManagerBase{
         session.setPrimarySession(false);
         session.setCreationTime(msg.getTimestamp());
         // use container maxInactiveInterval so that session will expire correctly in case of primary transfer
-        session.setMaxInactiveInterval(((Context) getContainer()).getSessionTimeout() * 60, false);
+        session.setMaxInactiveInterval(getMaxInactiveInterval(), false);
         session.access();
         session.setId(msg.getSessionID(), notifySessionListenersOnReplication);
         session.resetDeltaRequest();
@@ -1640,12 +1357,14 @@ public class DeltaManager extends ClusterManagerBase{
             sendSessions(sender, currentSessions, findSessionTimestamp);
         } else {
             // send session at blocks
+            int remain = currentSessions.length;
             for (int i = 0; i < currentSessions.length; i += getSendAllSessionsSize()) {
                 int len = i + getSendAllSessionsSize() > currentSessions.length ? currentSessions.length - i : getSendAllSessionsSize();
                 Session[] sendSessions = new Session[len];
                 System.arraycopy(currentSessions, i, sendSessions, 0, len);
                 sendSessions(sender, sendSessions,findSessionTimestamp);
-                if (getSendAllSessionsWaitTime() > 0) {
+                remain = remain - len;
+                if (getSendAllSessionsWaitTime() > 0 && remain > 0) {
                     try {
                         Thread.sleep(getSendAllSessionsWaitTime());
                     } catch (Exception sleep) {
@@ -1675,12 +1394,8 @@ public class DeltaManager extends ClusterManagerBase{
             session.setPrimarySession(false);
             session.setId(newSessionID, false);
             if (notifyContainerListenersOnReplication) {
-                Container c = getContainer();
-                if (c instanceof StandardContext) {
-                    ((StandardContext) getContainer()).fireContainerEvent(
-                            Context.CHANGE_SESSION_ID_EVENT,
-                            new String[] {msg.getSessionID(), newSessionID});
-                }
+                getContainer().fireContainerEvent(Context.CHANGE_SESSION_ID_EVENT,
+                        new String[] {msg.getSessionID(), newSessionID});
             }
         }
     }
@@ -1692,7 +1407,7 @@ public class DeltaManager extends ClusterManagerBase{
      */
     protected void handleALL_SESSION_NOCONTEXTMANAGER(SessionMessage msg, Member sender) {
         counterReceive_EVT_ALL_SESSION_NOCONTEXTMANAGER++ ;
-        if (log.isDebugEnabled()) 
+        if (log.isDebugEnabled())
             log.debug(sm.getString("deltaManager.receiveMessage.noContextManager",getName(), sender.getHost(), Integer.valueOf(sender.getPort())));
         noContextManagerReceived = true ;
     }
@@ -1714,25 +1429,18 @@ public class DeltaManager extends ClusterManagerBase{
         cluster.send(newmsg, sender);
     }
 
+    @Override
     public ClusterManager cloneFromTemplate() {
         DeltaManager result = new DeltaManager();
-        result.name = "Clone-from-"+name;
-        result.cluster = cluster;
-        result.replicationValve = replicationValve;
-        result.maxActiveSessions = maxActiveSessions;
+        clone(result);
         result.expireSessionsOnShutdown = expireSessionsOnShutdown;
-        result.notifyListenersOnReplication = notifyListenersOnReplication;
         result.notifySessionListenersOnReplication = notifySessionListenersOnReplication;
         result.notifyContainerListenersOnReplication = notifyContainerListenersOnReplication;
         result.stateTransferTimeout = stateTransferTimeout;
         result.sendAllSessions = sendAllSessions;
-        result.sendClusterDomainOnly = sendClusterDomainOnly ;
         result.sendAllSessionsSize = sendAllSessionsSize;
         result.sendAllSessionsWaitTime = sendAllSessionsWaitTime ; 
-        result.receiverQueue = receiverQueue ;
         result.stateTimestampDrop = stateTimestampDrop ;
-        result.stateTransferCreateSendTime = stateTransferCreateSendTime; 
-        result.setSessionAttributeFilter(getSessionAttributeFilter());
         return result;
     }
 }
