@@ -14,7 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.catalina.ha.session;
 
 import java.io.Externalizable;
@@ -176,6 +175,7 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
                     Thread.currentThread().setContextClassLoader(loaders[0]);
                 getDeltaRequest().readExternal(stream);
                 getDeltaRequest().execute(this, ((ClusterManager)getManager()).isNotifyListenersOnReplication());
+                stream.close();
             } finally {
                 Thread.currentThread().setContextClassLoader(contextLoader);
             }
@@ -514,6 +514,41 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         return (sb.toString());
     }
 
+    @Override
+    public void addSessionListener(SessionListener listener) {
+        addSessionListener(listener, true);
+    }
+
+    public void addSessionListener(SessionListener listener, boolean addDeltaRequest) {
+        lock();
+        try {
+            super.addSessionListener(listener);
+            if (addDeltaRequest && deltaRequest != null && listener instanceof ReplicatedSessionListener) {
+                deltaRequest.addSessionListener(listener);
+            }
+        } finally {
+            unlock();
+        }
+    }
+
+    @Override
+    public void removeSessionListener(SessionListener listener) {
+        removeSessionListener(listener, true);
+    }
+
+    public void removeSessionListener(SessionListener listener, boolean addDeltaRequest) {
+        lock();
+        try {
+            super.removeSessionListener(listener);
+            if (addDeltaRequest && deltaRequest != null && listener instanceof ReplicatedSessionListener) {
+                deltaRequest.removeSessionListener(listener);
+            }
+        } finally {
+            unlock();
+        }
+    }
+
+
     // ------------------------------------------------ Session Package Methods
 
     @Override
@@ -571,7 +606,9 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         try {
             lock();
             if (deltaRequest == null) {
-                deltaRequest = new DeltaRequest(getIdInternal(), false);
+                boolean recordAllActions = manager instanceof ClusterManagerBase &&
+                        ((ClusterManagerBase)manager).isRecordAllActions();
+                deltaRequest = new DeltaRequest(getIdInternal(), recordAllActions);
             } else {
                 deltaRequest.reset();
                 deltaRequest.setSessionId(getIdInternal());
@@ -755,10 +792,14 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
         }
         isValid = isValidSave;
 
-        if (listeners == null) {
-            ArrayList<SessionListener> arrayList =
-                new ArrayList<SessionListener>();
-            listeners = arrayList;
+        // Session listeners
+        n = ((Integer) stream.readObject()).intValue();
+        if (listeners == null || n > 0) {
+            listeners = new ArrayList<SessionListener>();
+        }
+        for (int i = 0; i < n; i++) {
+            SessionListener listener = (SessionListener) stream.readObject();
+            listeners.add(listener);
         }
 
         if (notes == null) {
@@ -850,8 +891,20 @@ public class DeltaSession extends StandardSession implements Externalizable,Clus
             }
         }
 
+        // Serializable listeners
+        ArrayList<SessionListener> saveListeners = new ArrayList<SessionListener>();
+        for (SessionListener listener : listeners) {
+            if (listener instanceof ReplicatedSessionListener) {
+                saveListeners.add(listener);
+            }
+        }
+        stream.writeObject(Integer.valueOf(saveListeners.size()));
+        for (SessionListener listener : saveListeners) {
+            stream.writeObject(listener);
+        }
     }
 
+
     // -------------------------------------------------------- Private Methods
 
 
