@@ -20,28 +20,39 @@ package org.apache.catalina.mbeans;
 import java.io.IOException;
 import java.io.Serializable;
 import java.lang.management.ManagementFactory;
+import java.net.InetAddress;
 import java.net.MalformedURLException;
+import java.net.ServerSocket;
 import java.net.Socket;
+import java.net.UnknownHostException;
+import java.rmi.AlreadyBoundException;
 import java.rmi.RemoteException;
 import java.rmi.registry.LocateRegistry;
+import java.rmi.registry.Registry;
 import java.rmi.server.RMIClientSocketFactory;
 import java.rmi.server.RMIServerSocketFactory;
+import java.security.NoSuchAlgorithmException;
+import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.List;
+import java.util.Locale;
 
-import javax.management.MBeanServer;
 import javax.management.remote.JMXConnectorServer;
-import javax.management.remote.JMXConnectorServerFactory;
 import javax.management.remote.JMXServiceURL;
 import javax.management.remote.rmi.RMIConnectorServer;
+import javax.management.remote.rmi.RMIJRMPServerImpl;
+import javax.net.ssl.SSLContext;
+import javax.net.ssl.SSLServerSocket;
+import javax.net.ssl.SSLServerSocketFactory;
 import javax.rmi.ssl.SslRMIClientSocketFactory;
 import javax.rmi.ssl.SslRMIServerSocketFactory;
 
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleEvent;
 import org.apache.catalina.LifecycleListener;
-import org.apache.tomcat.util.res.StringManager;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.res.StringManager;
 
 /**
  * This listener fixes the port used by JMX/RMI Server making things much
@@ -52,18 +63,16 @@ import org.apache.juli.logging.LogFactory;
  */
 public class JmxRemoteLifecycleListener implements LifecycleListener {
     
-    private static Log log =
-        LogFactory.getLog(JmxRemoteLifecycleListener.class);
+    private static final Log log = LogFactory.getLog(JmxRemoteLifecycleListener.class);
     
-    /**
-     * The string resources for this package.
-     */
     protected static final StringManager sm =
-        StringManager.getManager(Constants.Package);
+            StringManager.getManager(Constants.Package);
 
+    protected String rmiBindAddress = null;
     protected int rmiRegistryPortPlatform = -1;
     protected int rmiServerPortPlatform = -1;
-    protected boolean rmiSSL = true;
+    protected boolean rmiRegistrySSL = true;
+    protected boolean rmiServerSSL = true;
     protected String ciphers[] = null;
     protected String protocols[] = null;
     protected boolean clientAuth = true;
@@ -75,6 +84,22 @@ public class JmxRemoteLifecycleListener implements LifecycleListener {
 
     protected JMXConnectorServer csPlatform = null;
     
+    /**
+     * Get the inet address on which the Platform RMI server is exported.
+     * @return The textual representation of inet address
+     */
+    public String getRmiBindAddress() {
+        return rmiBindAddress;
+    }
+
+    /**
+     * Set the inet address on which the Platform RMI server is exported.
+     * @param theRmiBindAddress The textual representation of inet address
+     */
+    public void setRmiBindAddress(String theRmiBindAddress) {
+        rmiBindAddress = theRmiBindAddress;
+    }
+
     /**
      * Get the port on which the Platform RMI server is exported. This is the
      * port that is normally chosen by the RMI stack.
@@ -134,9 +159,13 @@ public class JmxRemoteLifecycleListener implements LifecycleListener {
         // Get all the other parameters required from the standard system
         // properties. Only need to get the parameters that affect the creation
         // of the server port.
-        String rmiSSLValue = System.getProperty(
+        String rmiRegistrySSLValue = System.getProperty(
+                "com.sun.management.jmxremote.registry.ssl", "false");
+        rmiRegistrySSL = Boolean.parseBoolean(rmiRegistrySSLValue);
+
+        String rmiServerSSLValue = System.getProperty(
                 "com.sun.management.jmxremote.ssl", "true");
-        rmiSSL = Boolean.parseBoolean(rmiSSLValue);
+        rmiServerSSL = Boolean.parseBoolean(rmiServerSSLValue);
 
         String protocolsValue = System.getProperty(
                 "com.sun.management.jmxremote.ssl.enabled.protocols");
@@ -151,7 +180,7 @@ public class JmxRemoteLifecycleListener implements LifecycleListener {
         }
 
         String clientAuthValue = System.getProperty(
-            "com.sun.management.jmxremote.ssl.need.client.auth", "true");
+                "com.sun.management.jmxremote.ssl.need.client.auth", "true");
         clientAuth = Boolean.parseBoolean(clientAuthValue);
 
         String authenticateValue = System.getProperty(
@@ -171,6 +200,7 @@ public class JmxRemoteLifecycleListener implements LifecycleListener {
     }
     
 
+    @Override
     public void lifecycleEvent(LifecycleEvent event) {
         // When the server starts, configure JMX/RMI
         if (Lifecycle.START_EVENT == event.getType()) {
@@ -183,29 +213,64 @@ public class JmxRemoteLifecycleListener implements LifecycleListener {
             // Create the environment
             HashMap<String,Object> env = new HashMap<String,Object>();
 
-            RMIClientSocketFactory csf = null;
-            RMIServerSocketFactory ssf = null;
+            RMIClientSocketFactory registryCsf = null;
+            RMIServerSocketFactory registrySsf = null;
 
-            // Configure SSL for RMI connection if required
-            if (rmiSSL) {
-                csf = new SslRMIClientSocketFactory();
-                ssf = new SslRMIServerSocketFactory(ciphers, protocols,
-                            clientAuth);
+            RMIClientSocketFactory serverCsf = null;
+            RMIServerSocketFactory serverSsf = null;
+
+            // Configure registry socket factories
+            if (rmiRegistrySSL) {
+                registryCsf = new SslRMIClientSocketFactory();
+                if (rmiBindAddress == null) {
+                    registrySsf = new SslRMIServerSocketFactory(
+                            ciphers, protocols, clientAuth);
+                } else {
+                    registrySsf = new SslRmiServerBindSocketFactory(
+                            ciphers, protocols, clientAuth, rmiBindAddress);
+                }
+            } else {
+                if (rmiBindAddress != null) {
+                    registrySsf = new RmiServerBindSocketFactory(rmiBindAddress);
+                }
+            }
+
+            // Configure server socket factories
+            if (rmiServerSSL) {
+                serverCsf = new SslRMIClientSocketFactory();
+                if (rmiBindAddress == null) {
+                    serverSsf = new SslRMIServerSocketFactory(
+                            ciphers, protocols, clientAuth);
+                } else {
+                    serverSsf = new SslRmiServerBindSocketFactory(
+                            ciphers, protocols, clientAuth, rmiBindAddress);
+                }
+            } else {
+                if (rmiBindAddress != null) {
+                    serverSsf = new RmiServerBindSocketFactory(rmiBindAddress);
+                }
             }
             
+            // By default, the registry will pick an address to listen on.
+            // Setting this property overrides that and ensures it listens on
+            // the configured address.
+            if (rmiBindAddress != null) {
+                System.setProperty("java.rmi.server.hostname", rmiBindAddress);
+            }
+
             // Force the use of local ports if required
             if (useLocalPorts) {
-                csf = new RmiClientLocalhostSocketFactory(csf);
+                registryCsf = new RmiClientLocalhostSocketFactory(registryCsf);
+                serverCsf = new RmiClientLocalhostSocketFactory(serverCsf);
             }
 
             // Populate the env properties used to create the server
-            if (csf != null) {
-                env.put(RMIConnectorServer.RMI_CLIENT_SOCKET_FACTORY_ATTRIBUTE,
-                        csf);
+            if (serverCsf != null) {
+                env.put(RMIConnectorServer.RMI_CLIENT_SOCKET_FACTORY_ATTRIBUTE, serverCsf);
+                env.put("com.sun.jndi.rmi.factory.socket", registryCsf);
             }
-            if (ssf != null) {
-                env.put(RMIConnectorServer.RMI_SERVER_SOCKET_FACTORY_ATTRIBUTE,
-                        ssf);
+            if (serverSsf != null) {
+                env.put(RMIConnectorServer.RMI_SERVER_SOCKET_FACTORY_ATTRIBUTE, serverSsf);
             }
 
             // Configure authentication
@@ -215,24 +280,27 @@ public class JmxRemoteLifecycleListener implements LifecycleListener {
                 env.put("jmx.remote.x.login.config", loginModuleName);
             }
 
-
             // Create the Platform server
-            csPlatform = createServer("Platform", rmiRegistryPortPlatform,
-                    rmiServerPortPlatform, env,
-                    ManagementFactory.getPlatformMBeanServer());
+            csPlatform = createServer("Platform", rmiBindAddress, rmiRegistryPortPlatform,
+                    rmiServerPortPlatform, env, registryCsf, registrySsf, serverCsf, serverSsf);
             
         } else if (Lifecycle.STOP_EVENT == event.getType()) {
             destroyServer("Platform", csPlatform);
         }
     }
 
+
     private JMXConnectorServer createServer(String serverName,
-            int theRmiRegistryPort, int theRmiServerPort,
-            HashMap<String,Object> theEnv, MBeanServer theMBeanServer) {
+            String bindAddress, int theRmiRegistryPort, int theRmiServerPort,
+            HashMap<String,Object> theEnv,
+            RMIClientSocketFactory registryCsf, RMIServerSocketFactory registrySsf,
+            RMIClientSocketFactory serverCsf, RMIServerSocketFactory serverSsf) {
         
         // Create the RMI registry
+        Registry registry;
         try {
-            LocateRegistry.createRegistry(theRmiRegistryPort);
+            registry = LocateRegistry.createRegistry(
+                    theRmiRegistryPort, registryCsf, registrySsf);
         } catch (RemoteException e) {
             log.error(sm.getString(
                     "jmxRemoteLifecycleListener.createRegistryFailed",
@@ -240,29 +308,27 @@ public class JmxRemoteLifecycleListener implements LifecycleListener {
             return null;
         }
 
-        // Build the connection string with fixed ports
-        StringBuffer url = new StringBuffer();
-        url.append("service:jmx:rmi://localhost:");
-        url.append(theRmiServerPort);
-        url.append("/jndi/rmi://localhost:");
-        url.append(theRmiRegistryPort);
-        url.append("/jmxrmi");
+        if (bindAddress == null) {
+            bindAddress = "localhost";
+        }
+
+        String url = "service:jmx:rmi://" + bindAddress;
         JMXServiceURL serviceUrl;
         try {
             serviceUrl = new JMXServiceURL(url.toString());
         } catch (MalformedURLException e) {
-            log.error(sm.getString(
-                    "jmxRemoteLifecycleListener.invalidURL",
-                    serverName, url.toString()), e);
+            log.error(sm.getString("jmxRemoteLifecycleListener.invalidURL", serverName, url), e);
             return null;
         }
         
-        // Start the JMX server with the connection string
-        JMXConnectorServer cs = null;
+        RMIConnectorServer cs = null;
         try {
-            cs = JMXConnectorServerFactory.newJMXConnectorServer(
-                    serviceUrl, theEnv, theMBeanServer);
+            RMIJRMPServerImpl server = new RMIJRMPServerImpl(
+                    rmiServerPortPlatform, serverCsf, serverSsf, theEnv);
+            cs = new RMIConnectorServer(serviceUrl, theEnv, server,
+                    ManagementFactory.getPlatformMBeanServer());
             cs.start();
+            registry.bind("jmxrmi", server);
             log.info(sm.getString("jmxRemoteLifecycleListener.start",
                     Integer.toString(theRmiRegistryPort),
                     Integer.toString(theRmiServerPort), serverName));
@@ -270,10 +336,15 @@ public class JmxRemoteLifecycleListener implements LifecycleListener {
             log.error(sm.getString(
                     "jmxRemoteLifecycleListener.createServerFailed",
                     serverName), e);
+        } catch (AlreadyBoundException e) {
+            log.error(sm.getString(
+                    "jmxRemoteLifecycleListener.createServerFailed",
+                    serverName), e);
         }
         return cs;
     }
 
+
     private void destroyServer(String serverName,
             JMXConnectorServer theConnectorServer) {
         if (theConnectorServer != null) {
@@ -287,8 +358,12 @@ public class JmxRemoteLifecycleListener implements LifecycleListener {
         }
     }
 
+
     public static class RmiClientLocalhostSocketFactory
-    implements RMIClientSocketFactory, Serializable {
+            implements RMIClientSocketFactory, Serializable {
+
+        private static final long serialVersionUID = 1L;
+
         private static final String FORCED_HOST = "localhost";
 
         private RMIClientSocketFactory factory = null;
@@ -297,6 +372,7 @@ public class JmxRemoteLifecycleListener implements LifecycleListener {
             factory = theFactory;
         }
 
+        @Override
         public Socket createSocket(String host, int port) throws IOException {
             if (factory == null) {
                 return new Socket(FORCED_HOST, port);
@@ -304,7 +380,89 @@ public class JmxRemoteLifecycleListener implements LifecycleListener {
                 return factory.createSocket(FORCED_HOST, port);
             }
         }
+    }
 
-        
+
+    public static class RmiServerBindSocketFactory implements RMIServerSocketFactory {
+
+        private final InetAddress bindAddress;
+
+        public RmiServerBindSocketFactory(String address) {
+            InetAddress bindAddress = null;
+            try {
+                bindAddress = InetAddress.getByName(address);
+            } catch (UnknownHostException e) {
+                log.error(sm.getString(
+                        "jmxRemoteLifecycleListener.invalidRmiBindAddress", address), e);
+                // bind address will be null which means any/all local addresses
+                // which should be safe
+            }
+            this.bindAddress = bindAddress;
+        }
+
+        @Override
+        public ServerSocket createServerSocket(int port) throws IOException  {
+            return new ServerSocket(port, 0, bindAddress);
+        }
+    }
+
+
+    public static class SslRmiServerBindSocketFactory extends SslRMIServerSocketFactory {
+
+        private static final SSLServerSocketFactory sslServerSocketFactory;
+        private static final String[] defaultProtocols;
+
+        static {
+            SSLContext sslContext;
+            try {
+                sslContext = SSLContext.getDefault();
+            } catch (NoSuchAlgorithmException e) {
+                // Can't continue. Force a failure.
+                throw new IllegalStateException(e);
+            }
+            sslServerSocketFactory = sslContext.getServerSocketFactory();
+            String[] protocols = sslContext.getDefaultSSLParameters().getProtocols();
+            List<String> filteredProtocols = new ArrayList<String>(protocols.length);
+            for (String protocol : protocols) {
+                if (protocol.toUpperCase(Locale.ENGLISH).contains("SSL")) {
+                    continue;
+                }
+                filteredProtocols.add(protocol);
+            }
+            defaultProtocols = filteredProtocols.toArray(new String[filteredProtocols.size()]);
+        }
+
+        private final InetAddress bindAddress;
+
+        public SslRmiServerBindSocketFactory(String[] enabledCipherSuites,
+                String[] enabledProtocols, boolean needClientAuth, String address) {
+            super(enabledCipherSuites, enabledProtocols, needClientAuth);
+            InetAddress bindAddress = null;
+            try {
+                bindAddress = InetAddress.getByName(address);
+            } catch (UnknownHostException e) {
+                log.error(sm.getString(
+                        "jmxRemoteLifecycleListener.invalidRmiBindAddress", address), e);
+                // bind address will be null which means any/all local addresses
+                // which should be safe
+            }
+            this.bindAddress = bindAddress;
+        }
+
+        @Override
+        public ServerSocket createServerSocket(int port) throws IOException  {
+            SSLServerSocket sslServerSocket =
+                    (SSLServerSocket) sslServerSocketFactory.createServerSocket(port, 0, bindAddress);
+            if (getEnabledCipherSuites() != null) {
+                sslServerSocket.setEnabledCipherSuites(getEnabledCipherSuites());
+            }
+            if (getEnabledProtocols() == null) {
+                sslServerSocket.setEnabledProtocols(defaultProtocols);
+            } else {
+                sslServerSocket.setEnabledProtocols(getEnabledProtocols());
+            }
+            sslServerSocket.setNeedClientAuth(getNeedClientAuth());
+            return sslServerSocket;
+        }
     }
 }
