@@ -37,18 +37,17 @@ import java.util.Locale;
 import java.util.StringTokenizer;
 import java.util.Vector;
 
+import javax.servlet.RequestDispatcher;
 import javax.servlet.ServletConfig;
 import javax.servlet.ServletContext;
 import javax.servlet.ServletException;
 import javax.servlet.ServletOutputStream;
-import javax.servlet.UnavailableException;
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import javax.servlet.http.HttpSession;
 
-import org.apache.catalina.Globals;
 import org.apache.catalina.util.IOTools;
 
 
@@ -180,7 +179,7 @@ import org.apache.catalina.util.IOTools;
  * </p>
  * <p>
  *
- * <b>Metavariable Values</b>: According to the CGI specificion,
+ * <b>Metavariable Values</b>: According to the CGI specification,
  * implementations may choose to represent both null or missing values in an
  * implementation-specific manner, but must define that manner.  This
  * implementation chooses to always define all required metavariables, but
@@ -193,7 +192,7 @@ import org.apache.catalina.util.IOTools;
  *
  * <b>NPH --  Non-parsed-header implementation</b>:  This implementation does
  * not support the CGI NPH concept, whereby server ensures that the data
- * supplied to the script are preceisely as supplied by the client and
+ * supplied to the script are precisely as supplied by the client and
  * unaltered by the server.
  * </p>
  * <p>
@@ -228,33 +227,21 @@ import org.apache.catalina.util.IOTools;
  * <li> Better documentation
  * <li> Confirm use of ServletInputStream.available() in CGIRunner.run() is
  *      not needed
- * <li> Make checking for "." and ".." in servlet & cgi PATH_INFO less
- *      draconian
  * <li> [add more to this TODO list]
  * </ul>
  * </p>
  *
  * @author Martin T Dengler [root@martindengler.com]
  * @author Amy Roh
- *
  * @since Tomcat 4.0
- *
  */
-
-
 public final class CGIServlet extends HttpServlet {
 
     /* some vars below copied from Craig R. McClanahan's InvokerServlet */
 
-    /**
-     * The debugging detail level for this servlet. Useful values range from 0
-     * to 5 where 0 means no logging and 5 means maximum logging. Values of 10
-     * or more mean maximum logging and debug info added to the HTTP response.
-     * If an error occurs and debug is 10 or more the standard error page
-     * mechanism will be disabled and a response body with debug information
-     * will be produced. Note that any value of 10 or more has the same effect
-     * as a value of 10.
-     */
+    private static final long serialVersionUID = 1L;
+
+    /** the debugging detail level for this servlet. */
     private int debug = 0;
 
     /**
@@ -302,18 +289,11 @@ public final class CGIServlet extends HttpServlet {
      *                                  interferes with the servlet's normal
      *                                  operation
      */
+    @Override
     public void init(ServletConfig config) throws ServletException {
 
         super.init(config);
 
-        // Verify that we were not accessed using the invoker servlet
-        String servletName = getServletConfig().getServletName();
-        if (servletName == null)
-            servletName = "";
-        if (servletName.startsWith("org.apache.catalina.INVOKER."))
-            throw new UnavailableException
-                ("Cannot invoke CGIServlet through the invoker");
-
         // Set our properties from the initialization parameters
         if (getServletConfig().getInitParameter("debug") != null)
             debug = Integer.parseInt(getServletConfig().getInitParameter("debug"));
@@ -354,7 +334,6 @@ public final class CGIServlet extends HttpServlet {
     }
 
 
-
     /**
      * Prints out important Servlet API and container information
      *
@@ -371,14 +350,15 @@ public final class CGIServlet extends HttpServlet {
      *
      */
     protected void printServletEnvironment(ServletOutputStream out,
-        HttpServletRequest req, HttpServletResponse res) throws IOException {
+        HttpServletRequest req, HttpServletResponse res)
+    throws IOException {
 
         // Document the properties from ServletRequest
         out.println("<h1>ServletRequest Properties</h1>");
         out.println("<ul>");
-        Enumeration attrs = req.getAttributeNames();
+        Enumeration<String> attrs = req.getAttributeNames();
         while (attrs.hasMoreElements()) {
-            String attr = (String) attrs.nextElement();
+            String attr = attrs.nextElement();
             out.println("<li><b>attribute</b> " + attr + " = " +
                            req.getAttribute(attr));
         }
@@ -388,14 +368,14 @@ public final class CGIServlet extends HttpServlet {
                        req.getContentLength());
         out.println("<li><b>contentType</b> = " +
                        req.getContentType());
-        Enumeration locales = req.getLocales();
+        Enumeration<Locale> locales = req.getLocales();
         while (locales.hasMoreElements()) {
-            Locale locale = (Locale) locales.nextElement();
+            Locale locale = locales.nextElement();
             out.println("<li><b>locale</b> = " + locale);
         }
-        Enumeration params = req.getParameterNames();
+        Enumeration<String> params = req.getParameterNames();
         while (params.hasMoreElements()) {
-            String param = (String) params.nextElement();
+            String param = params.nextElement();
             String values[] = req.getParameterValues(param);
             for (int i = 0; i < values.length; i++)
                 out.println("<li><b>parameter</b> " + param + " = " +
@@ -422,9 +402,9 @@ public final class CGIServlet extends HttpServlet {
             for (int i = 0; i < cookies.length; i++)
                 out.println("<li><b>cookie</b> " + cookies[i].getName() +" = " +cookies[i].getValue());
         }
-        Enumeration headers = req.getHeaderNames();
+        Enumeration<String> headers = req.getHeaderNames();
         while (headers.hasMoreElements()) {
-            String header = (String) headers.nextElement();
+            String header = headers.nextElement();
             out.println("<li><b>header</b> " + header + " = " +
                            req.getHeader(header));
         }
@@ -459,7 +439,7 @@ public final class CGIServlet extends HttpServlet {
         out.println("<ul>");
         attrs = req.getAttributeNames();
         while (attrs.hasMoreElements()) {
-            String attr = (String) attrs.nextElement();
+            String attr = attrs.nextElement();
             out.println("<li><b>" + attr + "</b> = " +
                            req.getAttribute(attr));
         }
@@ -489,7 +469,7 @@ public final class CGIServlet extends HttpServlet {
             out.println("<ul>");
             attrs = session.getAttributeNames();
             while (attrs.hasMoreElements()) {
-                String attr = (String) attrs.nextElement();
+                String attr = attrs.nextElement();
                 out.println("<li><b>" + attr + "</b> = " +
                                session.getAttribute(attr));
             }
@@ -511,7 +491,7 @@ public final class CGIServlet extends HttpServlet {
         out.println("<ul>");
         params = getServletConfig().getInitParameterNames();
         while (params.hasMoreElements()) {
-            String param = (String) params.nextElement();
+            String param = params.nextElement();
             String value = getServletConfig().getInitParameter(param);
             out.println("<li><b>" + param + "</b> = " + value);
         }
@@ -537,7 +517,7 @@ public final class CGIServlet extends HttpServlet {
         out.println("<ul>");
         params = getServletContext().getInitParameterNames();
         while (params.hasMoreElements()) {
-            String param = (String) params.nextElement();
+            String param = params.nextElement();
             String value = getServletContext().getInitParameter(param);
             out.println("<li><b>" + param + "</b> = " + value);
         }
@@ -549,7 +529,7 @@ public final class CGIServlet extends HttpServlet {
         out.println("<ul>");
         attrs = getServletContext().getAttributeNames();
         while (attrs.hasMoreElements()) {
-            String attr = (String) attrs.nextElement();
+            String attr = attrs.nextElement();
             out.println("<li><b>" + attr + "</b> = " +
                            getServletContext().getAttribute(attr));
         }
@@ -557,11 +537,9 @@ public final class CGIServlet extends HttpServlet {
         out.println("<hr>");
 
 
-
     }
 
 
-
     /**
      * Provides CGI Gateway service -- delegates to <code>doGet</code>
      *
@@ -574,13 +552,13 @@ public final class CGIServlet extends HttpServlet {
      * @see javax.servlet.http.HttpServlet
      *
      */
+    @Override
     protected void doPost(HttpServletRequest req, HttpServletResponse res)
         throws IOException, ServletException {
         doGet(req, res);
     }
 
 
-
     /**
      * Provides CGI Gateway service
      *
@@ -593,14 +571,10 @@ public final class CGIServlet extends HttpServlet {
      * @see javax.servlet.http.HttpServlet
      *
      */
+    @Override
     protected void doGet(HttpServletRequest req, HttpServletResponse res)
         throws ServletException, IOException {
 
-        // Verify that we were not accessed using the invoker servlet
-        if (req.getAttribute(Globals.INVOKED_ATTR) != null)
-            throw new UnavailableException
-                ("Cannot invoke CGIServlet through the invoker");
-
         CGIEnvironment cgiEnv = new CGIEnvironment(req, getServletContext());
 
         if (cgiEnv.isValid()) {
@@ -618,9 +592,7 @@ public final class CGIServlet extends HttpServlet {
         }
 
         if (!cgiEnv.isValid()) {
-            if (setStatus(res, 404)) {
-                return;
-            }
+            res.setStatus(404);
         }
 
         if (debug >= 10) {
@@ -663,29 +635,6 @@ public final class CGIServlet extends HttpServlet {
     } //doGet
 
 
-    /*
-     * Behaviour depends on the status code and the value of debug.
-     *
-     * Status < 400  - Always calls setStatus. Returns false. CGI servlet will
-     *                 provide the response body.
-     * Status >= 400 - Depends on debug
-     *   debug < 10    - Calls sendError(status), returns true. Standard error
-     *                   page mechanism will provide the response body.
-     *   debug >= 10   - Calls setStatus(status), return false. CGI servlet will
-     *                   provide the response body.
-     */
-    private boolean setStatus(HttpServletResponse response, int status) throws IOException {
-
-        if (status >= HttpServletResponse.SC_BAD_REQUEST && debug < 10) {
-            response.sendError(status);
-            return true;
-        } else {
-            response.setStatus(status);
-            return false;
-        }
-    }
-
-
     /**
      * Encapsulates the CGI environment and rules to derive
      * that environment from the servlet container and request information.
@@ -693,9 +642,7 @@ public final class CGIServlet extends HttpServlet {
      * <p>
      * </p>
      *
-     *
      * @since    Tomcat 4.0
-     *
      */
     protected class CGIEnvironment {
 
@@ -719,7 +666,7 @@ public final class CGIServlet extends HttpServlet {
         private File tmpDir = null;
 
         /** derived cgi environment */
-        private Hashtable env = null;
+        private Hashtable<String, String> env = null;
 
         /** cgi command to be invoked */
         private String command = null;
@@ -759,7 +706,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Uses the ServletContext to set some CGI variables
          *
@@ -769,11 +715,10 @@ public final class CGIServlet extends HttpServlet {
         protected void setupFromContext(ServletContext context) {
             this.context = context;
             this.webAppRootDir = context.getRealPath("/");
-            this.tmpDir = (File) context.getAttribute(Globals.WORK_DIR_ATTR);
+            this.tmpDir = (File) context.getAttribute(ServletContext.TEMPDIR);
         }
 
 
-
         /**
          * Uses the HttpServletRequest to set most CGI variables
          *
@@ -787,16 +732,17 @@ public final class CGIServlet extends HttpServlet {
             boolean isIncluded = false;
 
             // Look to see if this request is an include
-            if (req.getAttribute(Globals.INCLUDE_REQUEST_URI_ATTR) != null) {
+            if (req.getAttribute(
+                    RequestDispatcher.INCLUDE_REQUEST_URI) != null) {
                 isIncluded = true;
             }
             if (isIncluded) {
                 this.contextPath = (String) req.getAttribute(
-                        Globals.INCLUDE_CONTEXT_PATH_ATTR);
+                        RequestDispatcher.INCLUDE_CONTEXT_PATH);
                 this.servletPath = (String) req.getAttribute(
-                        Globals.INCLUDE_SERVLET_PATH_ATTR);
+                        RequestDispatcher.INCLUDE_SERVLET_PATH);
                 this.pathInfo = (String) req.getAttribute(
-                        Globals.INCLUDE_PATH_INFO_ATTR);
+                        RequestDispatcher.INCLUDE_PATH_INFO);
             } else {
                 this.contextPath = req.getContextPath();
                 this.servletPath = req.getServletPath();
@@ -818,7 +764,7 @@ public final class CGIServlet extends HttpServlet {
                 String qs;
                 if (isIncluded) {
                     qs = (String) req.getAttribute(
-                            Globals.INCLUDE_QUERY_STRING_ATTR);
+                            RequestDispatcher.INCLUDE_QUERY_STRING);
                 } else {
                     qs = req.getQueryString();
                 }
@@ -939,21 +885,21 @@ public final class CGIServlet extends HttpServlet {
             }
             if (!currentLocation.isFile()) {
                 return new String[] { null, null, null, null };
-            } else {
-                if (debug >= 2) {
-                    log("findCGI: FOUND cgi at " + currentLocation);
-                }
-                path = currentLocation.getAbsolutePath();
-                name = currentLocation.getName();
+            }
 
-                if (".".equals(contextPath)) {
-                    scriptname = servletPath;
-                } else {
-                    scriptname = contextPath + servletPath;
-                }
-                if (!servletPath.equals(cginame)) {
-                    scriptname = scriptname + cginame;
-                }
+            if (debug >= 2) {
+                log("findCGI: FOUND cgi at " + currentLocation);
+            }
+            path = currentLocation.getAbsolutePath();
+            name = currentLocation.getName();
+
+            if (".".equals(contextPath)) {
+                scriptname = servletPath;
+            } else {
+                scriptname = contextPath + servletPath;
+            }
+            if (!servletPath.equals(cginame)) {
+                scriptname = scriptname + cginame;
             }
 
             if (debug >= 1) {
@@ -965,10 +911,10 @@ public final class CGIServlet extends HttpServlet {
 
         /**
          * Constructs the CGI environment to be supplied to the invoked CGI
-         * script; relies heavliy on Servlet API methods and findCGI
+         * script; relies heavily on Servlet API methods and findCGI
          *
          * @param    req request associated with the CGI
-         *           invokation
+         *           Invocation
          *
          * @return   true if environment was set OK, false if there
          *           was a problem and no environment was set
@@ -988,7 +934,6 @@ public final class CGIServlet extends HttpServlet {
 
             // Add the CGI environment variables
             String sPathInfoOrig = null;
-            String sPathTranslatedOrig = null;
             String sPathInfoCGI = null;
             String sPathTranslatedCGI = null;
             String sCGIFullPath = null;
@@ -1001,10 +946,6 @@ public final class CGIServlet extends HttpServlet {
             sPathInfoOrig = this.pathInfo;
             sPathInfoOrig = sPathInfoOrig == null ? "" : sPathInfoOrig;
 
-            sPathTranslatedOrig = req.getPathTranslated();
-            sPathTranslatedOrig =
-                sPathTranslatedOrig == null ? "" : sPathTranslatedOrig;
-
             if (webAppRootDir == null ) {
                 // The app has not been deployed in exploded form
                 webAppRootDir = tmpDir.toString();
@@ -1038,7 +979,8 @@ public final class CGIServlet extends HttpServlet {
             envp.put("SERVER_PROTOCOL", nullsToBlanks(req.getProtocol()));
 
             int port = req.getServerPort();
-            Integer iPort = (port == 0 ? new Integer(-1) : new Integer(port));
+            Integer iPort =
+                (port == 0 ? Integer.valueOf(-1) : Integer.valueOf(port));
             envp.put("SERVER_PORT", iPort.toString());
 
             envp.put("REQUEST_METHOD", nullsToBlanks(req.getMethod()));
@@ -1087,8 +1029,6 @@ public final class CGIServlet extends HttpServlet {
              */
             if (sPathInfoCGI != null && !("".equals(sPathInfoCGI))) {
                 sPathTranslatedCGI = context.getRealPath(sPathInfoCGI);
-            } else {
-                sPathTranslatedCGI = null;
             }
             if (sPathTranslatedCGI == null || "".equals(sPathTranslatedCGI)) {
                 //NOOP
@@ -1120,15 +1060,15 @@ public final class CGIServlet extends HttpServlet {
              */
             int contentLength = req.getContentLength();
             String sContentLength = (contentLength <= 0 ? "" :
-                                     (new Integer(contentLength)).toString());
+                (Integer.valueOf(contentLength)).toString());
             envp.put("CONTENT_LENGTH", sContentLength);
 
 
-            Enumeration headers = req.getHeaderNames();
+            Enumeration<String> headers = req.getHeaderNames();
             String header = null;
             while (headers.hasMoreElements()) {
                 header = null;
-                header = ((String) headers.nextElement()).toUpperCase();
+                header = headers.nextElement().toUpperCase(Locale.ENGLISH);
                 //REMIND: rewrite multiple headers as if received as single
                 //REMIND: change character set
                 //REMIND: I forgot what the previous REMIND means
@@ -1159,8 +1099,8 @@ public final class CGIServlet extends HttpServlet {
          * directory to enable CGI script to be executed.
          */
         protected void expandCGIScript() {
-            StringBuffer srcPath = new StringBuffer();
-            StringBuffer destPath = new StringBuffer();
+            StringBuilder srcPath = new StringBuilder();
+            StringBuilder destPath = new StringBuilder();
             InputStream is = null;
 
             // paths depend on mapping
@@ -1195,15 +1135,26 @@ public final class CGIServlet extends HttpServlet {
 
             File f = new File(destPath.toString());
             if (f.exists()) {
+                try {
+                    is.close();
+                } catch (IOException e) {
+                    log("Could not close is", e);
+                }
                 // Don't need to expand if it already exists
                 return;
             }
 
             // create directories
-            String dirPath = new String (destPath.toString().substring(
-                    0,destPath.toString().lastIndexOf("/")));
+            String dirPath = destPath.toString().substring(
+                    0,destPath.toString().lastIndexOf("/"));
             File dir = new File(dirPath);
-            dir.mkdirs();
+            if (!dir.mkdirs() && !dir.isDirectory()) {
+                if (debug >= 2) {
+                    log("expandCGIScript: failed to create directories for '" +
+                            dir.getAbsolutePath() + "'");
+                }
+                return;
+            }
 
             try {
                 synchronized (expandFileLock) {
@@ -1218,10 +1169,17 @@ public final class CGIServlet extends HttpServlet {
                     }
                     FileOutputStream fos = new FileOutputStream(f);
 
-                    // copy data
-                    IOTools.flow(is, fos);
-                    is.close();
-                    fos.close();
+                    try {
+                        // copy data
+                        IOTools.flow(is, fos);
+                    } finally {
+                        try {
+                            is.close();
+                        } catch (IOException e) {
+                            log("Could not close is.", e);
+                        }
+                        fos.close();
+                    }
                     if (debug >= 2) {
                         log("expandCGIScript: expanded '" + srcPath + "' to '" + destPath + "'");
                     }
@@ -1229,7 +1187,10 @@ public final class CGIServlet extends HttpServlet {
             } catch (IOException ioe) {
                 // delete in case file is corrupted
                 if (f.exists()) {
-                    f.delete();
+                    if (!f.delete() && debug >= 2) {
+                        log("expandCGIScript: failed to delete '" +
+                                f.getAbsolutePath() + "'");
+                    }
                 }
             }
         }
@@ -1242,9 +1203,10 @@ public final class CGIServlet extends HttpServlet {
          * @return  HTML string containing CGI environment info
          *
          */
+        @Override
         public String toString() {
 
-            StringBuffer sb = new StringBuffer();
+            StringBuilder sb = new StringBuilder();
 
             sb.append("<TABLE border=2>");
 
@@ -1260,13 +1222,13 @@ public final class CGIServlet extends HttpServlet {
             sb.append("</td></tr>");
 
             if (isValid()) {
-                Enumeration envk = env.keys();
+                Enumeration<String> envk = env.keys();
                 while (envk.hasMoreElements()) {
-                    String s = (String) envk.nextElement();
+                    String s = envk.nextElement();
                     sb.append("<tr><td>");
                     sb.append(s);
                     sb.append("</td><td>");
-                    sb.append(blanksToString((String) env.get(s),
+                    sb.append(blanksToString(env.get(s),
                                              "[will be set to blank]"));
                     sb.append("</td></tr>");
                 }
@@ -1286,7 +1248,7 @@ public final class CGIServlet extends HttpServlet {
 
             sb.append("<tr><td>Command Line Params</td><td>");
             for (int i=0; i < cmdLineParameters.size(); i++) {
-                String param = (String) cmdLineParameters.get(i);
+                String param = cmdLineParameters.get(i);
                 sb.append("<p>");
                 sb.append(param);
                 sb.append("</p>");
@@ -1299,7 +1261,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Gets derived command string
          *
@@ -1311,7 +1272,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Gets derived CGI working directory
          *
@@ -1323,31 +1283,28 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Gets derived CGI environment
          *
          * @return   CGI environment
          *
          */
-        protected Hashtable getEnvironment() {
+        protected Hashtable<String,String> getEnvironment() {
             return env;
         }
 
 
-
         /**
          * Gets derived CGI query parameters
          *
          * @return   CGI query parameters
          *
          */
-        protected ArrayList getParameters() {
+        protected ArrayList<String> getParameters() {
             return cmdLineParameters;
         }
 
 
-
         /**
          * Gets validity status
          *
@@ -1360,7 +1317,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Converts null strings to blank strings ("")
          *
@@ -1373,7 +1329,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Converts null strings to another string
          *
@@ -1388,7 +1343,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Converts blank strings to another string
          *
@@ -1405,14 +1359,9 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
     } //class CGIEnvironment
 
 
-
-
-
-
     /**
      * Encapsulates the knowledge of how to run a CGI script, given the
      * script's desired environment and (optionally) input/output streams
@@ -1426,7 +1375,7 @@ public final class CGIServlet extends HttpServlet {
      * <p>
      *
      * The CGI environment and settings are derived from the information
-     * passed to the constuctor.
+     * passed to the constructor.
      *
      * </p>
      * <p>
@@ -1434,23 +1383,20 @@ public final class CGIServlet extends HttpServlet {
      * The input and output streams can be set by the <code>setInput</code>
      * and <code>setResponse</code> methods, respectively.
      * </p>
-     *
-     *
      */
-
     protected class CGIRunner {
 
         /** script/command to be executed */
         private String command = null;
 
         /** environment used when invoking the cgi script */
-        private Hashtable env = null;
+        private Hashtable<String,String> env = null;
 
         /** working directory used when invoking the cgi script */
         private File wd = null;
 
         /** command line parameters to be passed to the invoked script */
-        private ArrayList params = null;
+        private ArrayList<String> params = null;
 
         /** stdin to be passed to cgi script */
         private InputStream stdin = null;
@@ -1462,8 +1408,6 @@ public final class CGIServlet extends HttpServlet {
         private boolean readyToRun = false;
 
 
-
-
         /**
          *  Creates a CGIRunner and initializes its environment, working
          *  directory, and query parameters.
@@ -1476,10 +1420,10 @@ public final class CGIServlet extends HttpServlet {
          * @param  env      Hashtable with the desired script environment
          * @param  wd       File with the script's desired working directory
          * @param  params   ArrayList with the script's query command line
-         *                  paramters as strings
+         *                  parameters as strings
          */
-        protected CGIRunner(String command, Hashtable env, File wd,
-                            ArrayList params) {
+        protected CGIRunner(String command, Hashtable<String,String> env,
+                            File wd, ArrayList<String> params) {
             this.command = command;
             this.env = env;
             this.wd = wd;
@@ -1488,7 +1432,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Checks & sets ready status
          */
@@ -1505,7 +1448,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Gets ready status
          *
@@ -1517,7 +1459,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Sets HttpServletResponse object used to set headers and send
          * output to
@@ -1531,7 +1472,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Sets standard input to be passed on to the invoked cgi script
          *
@@ -1544,7 +1484,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Converts a Hashtable to a String array by converting each
          * key/value pair in the Hashtable to a String in the form
@@ -1557,13 +1496,13 @@ public final class CGIServlet extends HttpServlet {
          * @exception  NullPointerException   if a hash key has a null value
          *
          */
-        protected String[] hashToStringArray(Hashtable h)
+        protected String[] hashToStringArray(Hashtable<String,?> h)
             throws NullPointerException {
             Vector<String> v = new Vector<String>();
-            Enumeration e = h.keys();
+            Enumeration<String> e = h.keys();
             while (e.hasMoreElements()) {
-                String k = e.nextElement().toString();
-                v.add(k + "=" + h.get(k));
+                String k = e.nextElement();
+                v.add(k + "=" + h.get(k).toString());
             }
             String[] strArr = new String[v.size()];
             v.copyInto(strArr);
@@ -1571,7 +1510,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Executes a CGI script with the desired environment, current working
          * directory, and input/output streams
@@ -1604,7 +1542,10 @@ public final class CGIServlet extends HttpServlet {
          *             segments</u>:
          *             This implementation does not allow "<code>.</code>" and
          *             "<code>..</code>" in the the path, and such characters
-         *             will result in an IOException being thrown;
+         *             will result in an IOException being thrown (this should
+         *             never happen since Tomcat normalises the requestURI
+         *             before determining the contextPath, servletPath and
+         *             pathInfo);
          *     <LI> <u>Implementation limitations</u>: This implementation
          *             does not impose any limitations except as documented
          *             above.  This implementation may be limited by the
@@ -1646,11 +1587,11 @@ public final class CGIServlet extends HttpServlet {
             }
 
             /* original content/structure of this section taken from
-             * http://developer.java.sun.com/developer/
-             *                               bugParade/bugs/4216884.html
+             * http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4216884
              * with major modifications by Martin Dengler
              */
             Runtime rt = null;
+            BufferedReader cgiHeaderReader = null;
             InputStream cgiOutput = null;
             BufferedReader commandsStdErr = null;
             Thread errReaderThread = null;
@@ -1674,7 +1615,7 @@ public final class CGIServlet extends HttpServlet {
                         cmdAndArgs.toArray(new String[cmdAndArgs.size()]),
                         hashToStringArray(env), wd);
 
-                String sContentLength = (String) env.get("CONTENT_LENGTH");
+                String sContentLength = env.get("CONTENT_LENGTH");
 
                 if(!"".equals(sContentLength)) {
                     commandsStdIn = new BufferedOutputStream(proc.getOutputStream());
@@ -1685,8 +1626,7 @@ public final class CGIServlet extends HttpServlet {
 
                 /* we want to wait for the process to exit,  Process.waitFor()
                  * is useless in our situation; see
-                 * http://developer.java.sun.com/developer/
-                 *                               bugParade/bugs/4223650.html
+                 * http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4223650
                  */
 
                 boolean isRunning = true;
@@ -1695,23 +1635,18 @@ public final class CGIServlet extends HttpServlet {
                 final BufferedReader stdErrRdr = commandsStdErr ;
 
                 errReaderThread = new Thread() {
+                    @Override
                     public void run () {
                         sendToLog(stdErrRdr) ;
-                    } ;
+                    }
                 };
                 errReaderThread.start();
 
                 InputStream cgiHeaderStream =
                     new HTTPHeaderInputStream(proc.getInputStream());
-                BufferedReader cgiHeaderReader =
+                cgiHeaderReader =
                     new BufferedReader(new InputStreamReader(cgiHeaderStream));
 
-                // Need to be careful here. If sendError() is called the
-                // response body should be provided by the standard error page
-                // process. But, if the output of the CGI process isn't read
-                // then that process can hang.
-                boolean skipBody = false;
-
                 while (isRunning) {
                     try {
                         //set headers
@@ -1722,14 +1657,14 @@ public final class CGIServlet extends HttpServlet {
                                 log("runCGI: addHeader(\"" + line + "\")");
                             }
                             if (line.startsWith("HTTP")) {
-                                skipBody = setStatus(response, getSCFromHttpStatusLine(line));
+                                response.setStatus(getSCFromHttpStatusLine(line));
                             } else if (line.indexOf(":") >= 0) {
                                 String header =
                                     line.substring(0, line.indexOf(":")).trim();
                                 String value =
                                     line.substring(line.indexOf(":") + 1).trim();
                                 if (header.equalsIgnoreCase("status")) {
-                                    skipBody = setStatus(response, getSCFromCGIStatusHeader(value));
+                                    response.setStatus(getSCFromCGIStatusHeader(value));
                                 } else {
                                     response.addHeader(header , value);
                                 }
@@ -1745,7 +1680,7 @@ public final class CGIServlet extends HttpServlet {
                         cgiOutput = proc.getInputStream();
 
                         try {
-                            while (!skipBody && (bufRead = cgiOutput.read(bBuf)) != -1) {
+                            while ((bufRead = cgiOutput.read(bBuf)) != -1) {
                                 if (debug >= 4) {
                                     log("runCGI: output " + bufRead +
                                         " bytes of data");
@@ -1757,7 +1692,9 @@ public final class CGIServlet extends HttpServlet {
                             // such as a socket disconnect on the servlet side; otherwise, the
                             // external process could hang
                             if (bufRead != -1) {
-                                while ((bufRead = cgiOutput.read(bBuf)) != -1) {}
+                                while ((bufRead = cgiOutput.read(bBuf)) != -1) {
+                                    // NOOP - just read the data
+                                }
                             }
                         }
 
@@ -1769,15 +1706,25 @@ public final class CGIServlet extends HttpServlet {
                         try {
                             Thread.sleep(500);
                         } catch (InterruptedException ignored) {
+                            // Ignore
                         }
                     }
                 } //replacement for Process.waitFor()
+
             }
             catch (IOException e){
                 log ("Caught exception " + e);
                 throw e;
             }
             finally{
+                // Close the header reader
+                if (cgiHeaderReader != null) {
+                    try {
+                        cgiHeaderReader.close();
+                    } catch (IOException ioe) {
+                        log ("Exception closing header reader " + ioe);
+                    }
+                }
                 // Close the output stream if used
                 if (cgiOutput != null) {
                     try {
@@ -1878,11 +1825,11 @@ public final class CGIServlet extends HttpServlet {
                     rdr.close() ;
                 } catch (IOException ce) {
                     log("sendToLog error", ce) ;
-                } ;
-            } ;
+                }
+            }
             if ( lineCount > 0 && debug > 2) {
                 log("runCGI: " + lineCount + " lines received on stderr") ;
-            } ;
+            }
         }
     } //class CGIRunner
 
@@ -1891,7 +1838,7 @@ public final class CGIServlet extends HttpServlet {
      * upto and including the two blank lines terminating the headers. It
      * allows the content to be read using bytes or characters as appropriate.
      */
-    protected class HTTPHeaderInputStream extends InputStream {
+    protected static class HTTPHeaderInputStream extends InputStream {
         private static final int STATE_CHARACTER = 0;
         private static final int STATE_FIRST_CR = 1;
         private static final int STATE_FIRST_LF = 2;
@@ -1909,6 +1856,7 @@ public final class CGIServlet extends HttpServlet {
         /**
          * @see java.io.InputStream#read()
          */
+        @Override
         public int read() throws IOException {
             if (state == STATE_HEADER_END) {
                 return -1;
