@@ -5,20 +5,17 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.valves;
 
-
 import java.io.IOException;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
@@ -37,10 +34,8 @@ import org.apache.catalina.Store;
 import org.apache.catalina.connector.Request;
 import org.apache.catalina.connector.Response;
 import org.apache.catalina.session.PersistentManager;
-import org.apache.catalina.util.StringManager;
 import org.apache.tomcat.util.security.PrivilegedSetTccl;
 
-
 /**
  * Valve that implements per-request session persistence. It is intended to be
  * used with non-sticky load-balancers.
@@ -51,37 +46,32 @@ import org.apache.tomcat.util.security.PrivilegedSetTccl;
  *                              per session at any one time.
  *
  * @author Jean-Frederic Clere
- *
  */
-
 public class PersistentValve extends ValveBase {
 
-
-    /**
-     * The descriptive information related to this implementation.
-     */
-    private static final String info =
-        "org.apache.catalina.valves.PersistentValve/1.0";
-
-
-    /**
-     * The string manager for this package.
-     */
-    private static final StringManager sm =
-        StringManager.getManager(Constants.Package);
-
-
     // Saves a couple of calls to getClassLoader() on every request. Under high
     // load these calls took just long enough to appear as a hot spot (although
     // a very minor one) in a profiler.
     private static final ClassLoader MY_CLASSLOADER = PersistentValve.class.getClassLoader();
 
+    private volatile boolean clBindRequired;
+
+
+    //------------------------------------------------------ Constructor
+
+    public PersistentValve() {
+        super(true);
+    }
+
     
     // ----------------------------------------------------- Instance Variables
 
-    private volatile boolean clBindRequired;
+    /**
+     * The descriptive information related to this implementation.
+     */
+    private static final String info = "org.apache.catalina.valves.PersistentValve/1.0";
+
 
-    
     // ------------------------------------------------------------- Properties
 
     /**
@@ -92,7 +82,7 @@ public class PersistentValve extends ValveBase {
         return info;
     }
 
-    
+
     @Override
     public void setContainer(Container container) {
         super.setContainer(container);
@@ -124,33 +114,32 @@ public class PersistentValve extends ValveBase {
         // Select the Context to be used for this Request
         Context context = request.getContext();
         if (context == null) {
-            response.sendError
-                (HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
-                 sm.getString("standardHost.noContext"));
+            response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
+                    sm.getString("standardHost.noContext"));
             return;
         }
 
-        // Bind the context CL to the current thread
-        Thread.currentThread().setContextClassLoader
-            (context.getLoader().getClassLoader());
-
         // Update the session last access time for our session (if any)
         String sessionId = request.getRequestedSessionId();
         Manager manager = context.getManager();
-        if (sessionId != null && manager instanceof PersistentManager) {
+        if (sessionId != null &&  manager instanceof PersistentManager) {
             Store store = ((PersistentManager) manager).getStore();
             if (store != null) {
                 Session session = null;
                 try {
+                    bind(context);
                     session = store.load(sessionId);
                 } catch (Exception e) {
                     container.getLogger().error("deserializeError");
+                } finally {
+                    unbind();
                 }
                 if (session != null) {
                     if (!session.isValid() ||
                         isSessionStale(session, System.currentTimeMillis())) {
-                        if (container.getLogger().isDebugEnabled())
+                        if (container.getLogger().isDebugEnabled()) {
                             container.getLogger().debug("session swapped in is invalid or expired");
+                        }
                         session.expire();
                         store.remove(sessionId);
                     } else {
@@ -164,60 +153,68 @@ public class PersistentValve extends ValveBase {
                 }
             }
         }
-        if (container.getLogger().isDebugEnabled())
+        if (container.getLogger().isDebugEnabled()) {
             container.getLogger().debug("sessionId: " + sessionId);
+        }
 
         // Ask the next valve to process the request.
         getNext().invoke(request, response);
 
-        // Read the sessionid after the response.
-        // HttpSession hsess = hreq.getSession(false);
-        Session hsess;
-        try {
-            hsess = request.getSessionInternal();
-        } catch (Exception ex) {
-            hsess = null;
-        }
-        String newsessionId = null;
-        if (hsess!=null)
-            newsessionId = hsess.getIdInternal();
+        // If still processing async, don't try to store the session
+        if (!request.isAsync()) {
+            // Read the sessionid after the response.
+            // HttpSession hsess = hreq.getSession(false);
+            Session hsess;
+            try {
+                hsess = request.getSessionInternal(false);
+            } catch (Exception ex) {
+                hsess = null;
+            }
+            String newsessionId = null;
+            if (hsess!=null) {
+                newsessionId = hsess.getIdInternal();
+            }
 
-        if (container.getLogger().isDebugEnabled())
-            container.getLogger().debug("newsessionId: " + newsessionId);
-        if (newsessionId!=null) {
+            if (container.getLogger().isDebugEnabled()) {
+                container.getLogger().debug("newsessionId: " + newsessionId);
+            }
+            if (newsessionId!=null) {
             try {
                 bind(context);
-                /* store the session in the store and remove it from the manager */
-                if (manager instanceof PersistentManager) {
-                    Session session = manager.findSession(newsessionId);
-                    Store store = ((PersistentManager) manager).getStore();
-                    if (store != null && session!=null &&
-                        session.isValid() &&
-                        !isSessionStale(session, System.currentTimeMillis())) {
-                        // ((StandardSession)session).passivate();
-                        store.save(session);
-                        ((PersistentManager) manager).removeSuper(session);
-                        session.recycle();
+                    /* store the session and remove it from the manager */
+                    if (manager instanceof PersistentManager) {
+                        Session session = manager.findSession(newsessionId);
+                        Store store = ((PersistentManager) manager).getStore();
+                        if (store != null && session != null && session.isValid() &&
+                                !isSessionStale(session, System.currentTimeMillis())) {
+                            store.save(session);
+                            ((PersistentManager) manager).removeSuper(session);
+                            session.recycle();
+                        } else {
+                            if (container.getLogger().isDebugEnabled()) {
+                                container.getLogger().debug("newsessionId store: " +
+                                        store + " session: " + session +
+                                        " valid: " +
+                                        (session == null ? "N/A" : Boolean.toString(
+                                                session.isValid())) +
+                                        " stale: " + isSessionStale(session,
+                                                System.currentTimeMillis()));
+                            }
+                        }                    
                     } else {
-                        if (container.getLogger().isDebugEnabled())
-                            container.getLogger().debug("newsessionId store: " + store + " session: " +
-                                    session + 
-                                    (session == null ? "" :
-                                        " valid: " + session.isValid() +
-                                        " stale: " +
-                                    isSessionStale(session, System.currentTimeMillis())));
-    
+                        if (container.getLogger().isDebugEnabled()) {
+                            container.getLogger().debug("newsessionId Manager: " +
+                                    manager);
+                        }
                     }
-                } else {
-                    if (container.getLogger().isDebugEnabled())
-                        container.getLogger().debug("newsessionId Manager: " + manager);
+                } finally {
+                    unbind();
                 }
-            } finally {
-                unbind();
             }
         }
     }
 
+
     /**
      * Indicate whether the session has been idle for longer
      * than its expiration date as of the supplied time.
@@ -225,20 +222,22 @@ public class PersistentValve extends ValveBase {
      * FIXME: Probably belongs in the Session class.
      */
     protected boolean isSessionStale(Session session, long timeNow) {
- 
-        int maxInactiveInterval = session.getMaxInactiveInterval();
-        if (maxInactiveInterval >= 0) {
-            int timeIdle = // Truncate, do not round up
-                (int) ((timeNow - session.getLastAccessedTime()) / 1000L);
-            if (timeIdle >= maxInactiveInterval)
-                return true;
+
+        if (session != null) {
+            int maxInactiveInterval = session.getMaxInactiveInterval();
+            if (maxInactiveInterval >= 0) {
+                int timeIdle = // Truncate, do not round up
+                    (int) ((timeNow - session.getThisAccessedTime()) / 1000L);
+                if (timeIdle >= maxInactiveInterval) {
+                    return true;
+                }
+            }
         }
- 
+
         return false;
- 
     }
 
-    
+
     private void bind(Context context) {
         // Bind the context CL to the current thread
         if (clBindRequired && context.getLoader() != null) {
