@@ -1262,7 +1262,7 @@ class Parser implements TagConstants {
 
     /*
      * Parse for a template text string until '<' or "${" or "#{" is encountered,
-     * recognizing escape sequences "<\%", "\$", and "\#".
+     * recognizing escape sequences "<\%", "\${", and "\#{".
      */
     private void parseTemplateText(Node parent) throws JasperException {
 
@@ -1270,48 +1270,64 @@ class Parser implements TagConstants {
             return;
 
         CharArrayWriter ttext = new CharArrayWriter();
-        // Output the first character
-        int ch = reader.nextChar();
-        if (ch == '\\') {
-            reader.pushChar();
-        } else {
-            ttext.write(ch);
-        }
 
         while (reader.hasMoreInput()) {
-            int prev = ch;
-            ch = reader.nextChar();
+            int ch = reader.nextChar();
             if (ch == '<') {
-                reader.pushChar();
-                break;
-            } else if ((ch == '$' || ch == '#') && !pageInfo.isELIgnored()) {
-                if (!reader.hasMoreInput()) {
-                    ttext.write(ch);
-                    break;
+                // Check for "<\%"
+                if (reader.nextChar() == '\\') {
+                    if (reader.nextChar() == '%') {
+                        ttext.append('<');
+                        ttext.append('%');
+                    } else {
+                        reader.pushChar();
+                        reader.pushChar();
+                        if (ttext.size() == 0) {
+                            ttext.append('<');
+                        } else {
+                            reader.pushChar();
+                            break;
+                        }
+                    }
+                } else {
+                    reader.pushChar();
+                    if (ttext.size() == 0) {
+                        ttext.append('<');
+                    } else {
+                        reader.pushChar();
+                        break;
+                    }
                 }
+            } else if (ch == '\\' && !pageInfo.isELIgnored()) {
+                int next = reader.nextChar();
+                if (next == '$' || next == '#') {
+                    if (reader.nextChar() == '{') {
+                        ttext.write(next);
+                        ttext.append('{');
+                    } else {
+                        ttext.append('\\');
+                        ttext.write(next);
+                        reader.pushChar();
+                    }
+                } else {
+                    ttext.append('\\');
+                    reader.pushChar();
+                }
+            } else if ((ch == '$' || ch == '#' && !pageInfo.isDeferredSyntaxAllowedAsLiteral()) &&
+                    !pageInfo.isELIgnored()) {
                 if (reader.nextChar() == '{') {
                     reader.pushChar();
                     reader.pushChar();
                     break;
+                } else {
+                    reader.pushChar();
+                    ttext.write(ch);
                 }
+            } else {
                 ttext.write(ch);
-                reader.pushChar();
-                continue;
-            } else if (ch == '\\') {
-                if (!reader.hasMoreInput()) {
-                    ttext.write('\\');
-                    break;
-                }
-                char next = (char) reader.peekChar();
-                // Looking for \% or \$ or \#
-                if ((prev == '<' && next == '%') ||
-                        ((next == '$' || next == '#') &&
-                                !pageInfo.isELIgnored())) {
-                    ch = reader.nextChar();
-                }
             }
-            ttext.write(ch);
         }
+
         new Node.TemplateText(ttext.toString(), start, parent);
     }
 
