@@ -26,16 +26,22 @@ import javax.naming.NameNotFoundException;
  * properly populated exceptions.
  * 
  * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
- *
  */
 public class ImmutableNameNotFoundException
     extends NameNotFoundException {
 
-    public void appendRemainingComponent(String name) {}
-    public void appendRemainingName(Name name) {}
-    public void setRemainingName(Name name) {}
-    public void setResolverName(Name name) {}
-    public void setRootCause(Throwable e) {}
+    private static final long serialVersionUID = 1L;
+    
+    @Override
+    public void appendRemainingComponent(String name) {/*NOOP*/}
+    @Override
+    public void appendRemainingName(Name name) {/*NOOP*/}
+    @Override
+    public void setRemainingName(Name name) {/*NOOP*/}
+    @Override
+    public void setResolvedName(Name name) {/*NOOP*/}
+    @Override
+    public void setRootCause(Throwable e) {/*NOOP*/}
 
     @Override
     public synchronized Throwable fillInStackTrace() {
