@@ -14,23 +14,22 @@
  *  See the License for the specific language governing permissions and
  *  limitations under the License.
  */
-
 package org.apache.tomcat.util.buf;
 
 import java.io.CharConversionException;
 import java.io.IOException;
 
-/** 
+/**
  *  All URL decoding happens here. This way we can reuse, review, optimize
  *  without adding complexity to the buffers.
  *
  *  The conversion will modify the original buffer.
- * 
+ *
  *  @author Costin Manolache
  */
 public final class UDecoder {
-    
-    protected static final boolean ALLOW_ENCODED_SLASH = 
+
+    public static final boolean ALLOW_ENCODED_SLASH =
         Boolean.valueOf(System.getProperty("org.apache.tomcat.util.buf.UDecoder.ALLOW_ENCODED_SLASH", "false")).booleanValue();
 
     private static class DecodeException extends CharConversionException {
@@ -57,13 +56,15 @@ public final class UDecoder {
     private static final IOException EXCEPTION_SLASH = new DecodeException(
             "noSlash");
 
-    public UDecoder() 
+    public UDecoder()
     {
     }
 
     /** URLDecode, will modify the source.  Includes converting
      *  '+' to ' '.
+     * @deprecated Unused. Will be removed in Tomcat 8.0.x onwards.
      */
+    @Deprecated
     public void convert( ByteChunk mb )
         throws IOException
     {
@@ -79,10 +80,10 @@ public final class UDecoder {
         byte buff[]=mb.getBytes();
         int end=mb.getEnd();
 
-        int idx= ByteChunk.indexOf( buff, start, end, '%' );
+        int idx= ByteChunk.findByte( buff, start, end, (byte) '%' );
         int idx2=-1;
         if( query ) {
-            idx2= ByteChunk.indexOf( buff, start, (idx >= 0 ? idx : end), '+' );
+            idx2= ByteChunk.findByte( buff, start, (idx >= 0 ? idx : end), (byte) '+' );
         }
         if( idx<0 && idx2<0 ) {
             return;
@@ -107,9 +108,10 @@ public final class UDecoder {
                 }
                 byte b1= buff[j+1];
                 byte b2=buff[j+2];
-                if( !isHexDigit( b1 ) || ! isHexDigit(b2 ))
+                if( !isHexDigit( b1 ) || ! isHexDigit(b2 )) {
                     throw EXCEPTION_NOT_HEX_DIGIT;
-                
+                }
+
                 j+=2;
                 int res=x2c( b1, b2 );
                 if (noSlash && (res == '/')) {
@@ -120,7 +122,7 @@ public final class UDecoder {
         }
 
         mb.setEnd( idx );
-        
+
         return;
     }
 
@@ -129,7 +131,9 @@ public final class UDecoder {
 
     /** In-buffer processing - the buffer will be modified
      *  Includes converting  '+' to ' '.
+     * @deprecated Unused. Will be removed in Tomcat 8.0.x onwards.
      */
+    @Deprecated
     public void convert( CharChunk mb )
         throws IOException
     {
@@ -175,9 +179,10 @@ public final class UDecoder {
                 }
                 char b1= buff[j+1];
                 char b2=buff[j+2];
-                if( !isHexDigit( b1 ) || ! isHexDigit(b2 ))
+                if( !isHexDigit( b1 ) || ! isHexDigit(b2 )) {
                     throw EXCEPTION_NOT_HEX_DIGIT;
-                
+                }
+
                 j+=2;
                 int res=x2c( b1, b2 );
                 if (noSlash && (res == '/')) {
@@ -191,7 +196,9 @@ public final class UDecoder {
 
     /** URLDecode, will modify the source
      *  Includes converting  '+' to ' '.
+     * @deprecated Unused. Will be removed in Tomcat 8.0.x onwards.
      */
+    @Deprecated
     public void convert(MessageBytes mb)
         throws IOException
     {
@@ -203,11 +210,13 @@ public final class UDecoder {
     public void convert(MessageBytes mb, boolean query)
         throws IOException
     {
-        
+
         switch (mb.getType()) {
         case MessageBytes.T_STR:
             String strValue=mb.toString();
-            if( strValue==null ) return;
+            if( strValue==null ) {
+                return;
+            }
             try {
                 mb.setString( convert( strValue, query ));
             } catch (RuntimeException ex) {
@@ -226,7 +235,11 @@ public final class UDecoder {
     }
 
     // XXX Old code, needs to be replaced !!!!
-    // 
+    //
+    /**
+     * @deprecated Unused. Will be removed in Tomcat 8.0.x onwards.
+     */
+    @Deprecated
     public final String convert(String str)
     {
         return convert(str, true);
@@ -234,10 +247,13 @@ public final class UDecoder {
 
     public final String convert(String str, boolean query)
     {
-        if (str == null)  return  null;
-        
-        if( (!query || str.indexOf( '+' ) < 0) && str.indexOf( '%' ) < 0 )
+        if (str == null) {
+            return  null;
+        }
+
+        if( (!query || str.indexOf( '+' ) < 0) && str.indexOf( '%' ) < 0 ) {
             return str;
+        }
 
         final boolean noSlash = !(ALLOW_ENCODED_SLASH || query);
 
@@ -298,7 +314,7 @@ public final class UDecoder {
                  ( c>='a' && c<='f' ) ||
                  ( c>='A' && c<='F' ));
     }
-    
+
     private static int x2c( byte b1, byte b2 ) {
         int digit= (b1>='A') ? ( (b1 & 0xDF)-'A') + 10 :
             (b1 -'0');
