@@ -14,7 +14,6 @@
  *  See the License for the specific language governing permissions and
  *  limitations under the License.
  */
-
 package org.apache.tomcat.util.http;
 
 import java.io.IOException;
@@ -23,11 +22,12 @@ import java.nio.charset.Charset;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Enumeration;
-import java.util.HashMap;
+import java.util.LinkedHashMap;
 import java.util.Map;
 
 import org.apache.tomcat.util.buf.B2CConverter;
 import org.apache.tomcat.util.buf.ByteChunk;
+import org.apache.tomcat.util.buf.CharChunk;
 import org.apache.tomcat.util.buf.MessageBytes;
 import org.apache.tomcat.util.buf.UDecoder;
 import org.apache.tomcat.util.log.UserDataHelper;
@@ -49,8 +49,8 @@ public final class Parameters {
     protected static final StringManager sm =
         StringManager.getManager("org.apache.tomcat.util.http");
 
-    private final HashMap<String,ArrayList<String>> paramHashValues =
-        new HashMap<String,ArrayList<String>>();
+    private final Map<String,ArrayList<String>> paramHashValues =
+            new LinkedHashMap<String,ArrayList<String>>();
 
     private boolean didQueryParameters=false;
 
@@ -101,20 +101,24 @@ public final class Parameters {
         }
     }
 
+
     public boolean isParseFailed() {
         return parseFailedReason != null;
     }
 
+
     public FailReason getParseFailedReason() {
         return parseFailedReason;
     }
 
+
     public void setParseFailedReason(FailReason failReason) {
         if (this.parseFailedReason == null) {
             this.parseFailedReason = failReason;
         }
     }
 
+
     public void recycle() {
         parameterCount = 0;
         paramHashValues.clear();
@@ -124,10 +128,12 @@ public final class Parameters {
         parseFailedReason = null;
     }
 
+
     // -------------------- Data access --------------------
     // Access to the current name/values, no side effect ( processing ).
     // You must explicitly call handleQueryParameters and the post methods.
 
+    @Deprecated
     public void addParameterValues(String key, String[] newValues) {
         if (key == null) {
             return;
@@ -159,7 +165,6 @@ public final class Parameters {
         return Collections.enumeration(paramHashValues.keySet());
     }
 
-    // Shortcut.
     public String getParameter(String name ) {
         handleQueryParameters();
         ArrayList<String> values = paramHashValues.get(name);
@@ -176,12 +181,15 @@ public final class Parameters {
     /** Process the query string into parameters
      */
     public void handleQueryParameters() {
-        if( didQueryParameters ) return;
+        if( didQueryParameters ) {
+            return;
+        }
 
         didQueryParameters=true;
 
-        if( queryMB==null || queryMB.isNull() )
+        if( queryMB==null || queryMB.isNull() ) {
             return;
+        }
 
         if(log.isDebugEnabled()) {
             log.debug("Decoding query " + decodedQuery + " " +
@@ -198,8 +206,22 @@ public final class Parameters {
     }
 
 
-    private void addParam( String key, String value ) {
-        if( key==null ) return;
+    public void addParameter( String key, String value )
+            throws IllegalStateException {
+
+        if( key==null ) {
+            return;
+        }
+
+        parameterCount ++;
+        if (limit > -1 && parameterCount > limit) {
+            // Processing this parameter will push us over the limit. ISE is
+            // what Request.parseParts() uses for requests that are too big
+            setParseFailedReason(FailReason.TOO_MANY_PARAMETERS);
+            throw new IllegalStateException(sm.getString(
+                    "parameters.maxCountFail", Integer.valueOf(limit)));
+        }
+
         ArrayList<String> values = paramHashValues.get(key);
         if (values == null) {
             values = new ArrayList<String>(1);
@@ -217,9 +239,10 @@ public final class Parameters {
     // if needed
     ByteChunk tmpName=new ByteChunk();
     ByteChunk tmpValue=new ByteChunk();
-    private ByteChunk origName=new ByteChunk();
-    private ByteChunk origValue=new ByteChunk();
-    private static final String DEFAULT_ENCODING = "ISO-8859-1";
+    private final ByteChunk origName=new ByteChunk();
+    private final ByteChunk origValue=new ByteChunk();
+    CharChunk tmpNameC=new CharChunk(1024);
+    public static final String DEFAULT_ENCODING = "ISO-8859-1";
     private static final Charset DEFAULT_CHARSET =
         Charset.forName(DEFAULT_ENCODING);
 
@@ -232,12 +255,8 @@ public final class Parameters {
                                   Charset charset) {
 
         if(log.isDebugEnabled()) {
-            try {
-                log.debug(sm.getString("parameters.bytes",
-                        new String(bytes, start, len, DEFAULT_CHARSET.name())));
-            } catch (UnsupportedEncodingException uee) {
-                // Not possible. All JVMs must support ISO-8859-1
-            }
+            log.debug(sm.getString("parameters.bytes",
+                    new String(bytes, start, len, DEFAULT_CHARSET)));
         }
 
         int decodeFailCount = 0;
@@ -246,28 +265,6 @@ public final class Parameters {
         int end = start + len;
 
         while(pos < end) {
-            parameterCount ++;
-
-            if (limit > -1 && parameterCount > limit) {
-                setParseFailedReason(FailReason.TOO_MANY_PARAMETERS);
-                UserDataHelper.Mode logMode = maxParamCountLog.getNextMode();
-                if (logMode != null) {
-                    String message = sm.getString("parameters.maxCountFail",
-                            Integer.valueOf(limit));
-                    switch (logMode) {
-                        case INFO_THEN_DEBUG:
-                            message += sm.getString(
-                                    "parameters.maxCountFail.fallToDebug");
-                            //$FALL-THROUGH$
-                        case INFO:
-                            log.info(message);
-                            break;
-                        case DEBUG:
-                            log.debug(message);
-                    }
-                }
-                break;
-            }
             int nameStart = pos;
             int nameEnd = -1;
             int valueStart = -1;
@@ -327,15 +324,10 @@ public final class Parameters {
             }
 
             if (log.isDebugEnabled() && valueStart == -1) {
-                try {
-                    log.debug(sm.getString("parameters.noequal",
-                            Integer.valueOf(nameStart),
-                            Integer.valueOf(nameEnd),
-                            new String(bytes, nameStart, nameEnd-nameStart,
-                                    DEFAULT_CHARSET.name())));
-                } catch (UnsupportedEncodingException uee) {
-                    // Not possible. All JVMs must support ISO-8859-1
-                }
+                log.debug(sm.getString("parameters.noequal",
+                        Integer.valueOf(nameStart), Integer.valueOf(nameEnd),
+                        new String(bytes, nameStart, nameEnd-nameStart,
+                                DEFAULT_CHARSET)));
             }
 
             if (nameEnd <= nameStart ) {
@@ -350,15 +342,10 @@ public final class Parameters {
                 // &=foo&
                 UserDataHelper.Mode logMode = userDataLog.getNextMode();
                 if (logMode != null) {
-                    String extract = null;
+                    String extract;
                     if (valueEnd > nameStart) {
-                        try {
-                            extract = new String(bytes, nameStart,
-                                    valueEnd - nameStart,
-                                    DEFAULT_CHARSET.name());
-                        } catch (UnsupportedEncodingException uee) {
-                            // Not possible. All JVMs must support ISO-8859-1
-                        }
+                        extract = new String(bytes, nameStart, valueEnd
+                                - nameStart, DEFAULT_CHARSET);
                     } else {
                         extract = "";
                     }
@@ -425,7 +412,28 @@ public final class Parameters {
                     value = "";
                 }
 
-                addParam(name, value);
+                try {
+                    addParameter(name, value);
+                } catch (IllegalStateException ise) {
+                    // Hitting limit stops processing further params but does
+                    // not cause request to fail.
+                    UserDataHelper.Mode logMode = maxParamCountLog.getNextMode();
+                    if (logMode != null) {
+                        String message = ise.getMessage();
+                        switch (logMode) {
+                            case INFO_THEN_DEBUG:
+                                message += sm.getString(
+                                        "parameters.maxCountFail.fallToDebug");
+                                //$FALL-THROUGH$
+                            case INFO:
+                                log.info(message);
+                                break;
+                            case DEBUG:
+                                log.debug(message);
+                        }
+                    }
+                    break;
+                }
             } catch (IOException e) {
                 setParseFailedReason(FailReason.URL_DECODING);
                 decodeFailCount++;
@@ -488,11 +496,13 @@ public final class Parameters {
         if( urlDec==null ) {
             urlDec=new UDecoder();
         }
-        urlDec.convert(bc);
+        urlDec.convert(bc, true);
     }
 
     public void processParameters( MessageBytes data, String encoding ) {
-        if( data==null || data.isNull() || data.getLength() <= 0 ) return;
+        if( data==null || data.isNull() || data.getLength() <= 0 ) {
+            return;
+        }
 
         if( data.getType() != MessageBytes.T_BYTES ) {
             data.toBytes();
