@@ -16,40 +16,48 @@
  */
 package org.apache.tomcat.util.net;
 
+import java.io.EOFException;
 import java.io.IOException;
+import java.net.SocketTimeoutException;
 import java.nio.ByteBuffer;
 import java.nio.channels.SelectionKey;
+import java.nio.channels.Selector;
 import java.nio.channels.SocketChannel;
+
 import javax.net.ssl.SSLEngine;
 import javax.net.ssl.SSLEngineResult;
 import javax.net.ssl.SSLEngineResult.HandshakeStatus;
 import javax.net.ssl.SSLEngineResult.Status;
-import java.nio.channels.Selector;
-import org.apache.tomcat.util.MutableInteger;
+import javax.net.ssl.SSLException;
+
+import org.apache.juli.logging.Log;
+import org.apache.juli.logging.LogFactory;
 
 /**
- * 
+ *
  * Implementation of a secure socket channel
  * @author Filip Hanik
  * @version 1.0
  */
 
 public class SecureNioChannel extends NioChannel  {
-    
+
+    protected static final Log log = LogFactory.getLog(SecureNioChannel.class);
+
     protected ByteBuffer netInBuffer;
     protected ByteBuffer netOutBuffer;
-    
+
     protected SSLEngine sslEngine;
-    
-    protected boolean initHandshakeComplete = false;
-    protected HandshakeStatus initHandshakeStatus; //gets set by begin handshake
-    
+
+    protected boolean handshakeComplete = false;
+    protected HandshakeStatus handshakeStatus; //gets set by handshake
+
     protected boolean closed = false;
     protected boolean closing = false;
-    
+
     protected NioSelectorPool pool;
-    
-    public SecureNioChannel(SocketChannel channel, SSLEngine engine, 
+
+    public SecureNioChannel(SocketChannel channel, SSLEngine engine,
                             ApplicationBufferHandler bufHandler, NioSelectorPool pool) throws IOException {
         super(channel,bufHandler);
         this.sslEngine = engine;
@@ -58,35 +66,37 @@ public class SecureNioChannel extends NioChannel  {
         //allocate network buffers - TODO, add in optional direct non-direct buffers
         if ( netInBuffer == null ) netInBuffer = ByteBuffer.allocateDirect(netBufSize);
         if ( netOutBuffer == null ) netOutBuffer = ByteBuffer.allocateDirect(netBufSize);
-        
+
         //selector pool for blocking operations
         this.pool = pool;
-        
+
         //ensure that the application has a large enough read/write buffers
         //by doing this, we should not encounter any buffer overflow errors
         bufHandler.expand(bufHandler.getReadBuffer(), appBufSize);
         bufHandler.expand(bufHandler.getWriteBuffer(), appBufSize);
         reset();
     }
-    
+
     public void reset(SSLEngine engine) throws IOException {
         this.sslEngine = engine;
         reset();
     }
+    @Override
     public void reset() throws IOException {
         super.reset();
         netOutBuffer.position(0);
         netOutBuffer.limit(0);
         netInBuffer.position(0);
         netInBuffer.limit(0);
-        initHandshakeComplete = false;
+        handshakeComplete = false;
         closed = false;
         closing = false;
         //initiate handshake
         sslEngine.beginHandshake();
-        initHandshakeStatus = sslEngine.getHandshakeStatus();
+        handshakeStatus = sslEngine.getHandshakeStatus();
     }
-    
+
+    @Override
     public int getBufferSize() {
         int size = super.getBufferSize();
         size += netInBuffer!=null?netInBuffer.capacity():0;
@@ -94,24 +104,31 @@ public class SecureNioChannel extends NioChannel  {
         return size;
     }
 
-    
-//===========================================================================================    
+
+//===========================================================================================
 //                  NIO SSL METHODS
 //===========================================================================================
     /**
-     * returns true if the network buffer has 
-     * been flushed out and is empty
-     * @return boolean
+     * Flush the channel.
+     *
+     * @param block     Should a blocking write be used?
+     * @param s
+     * @param timeout
+     * @return <code>true</code> if the network buffer has been flushed out and
+     *         is empty else <code>false</code>
+     * @throws IOException
      */
-    public boolean flush(boolean block, Selector s, long timeout,MutableInteger lastWrite) throws IOException {
+    @Override
+    public boolean flush(boolean block, Selector s, long timeout)
+            throws IOException {
         if (!block) {
             flush(netOutBuffer);
         } else {
-            pool.write(netOutBuffer, this, s, timeout,block,lastWrite);
+            pool.write(netOutBuffer, this, s, timeout,block);
         }
         return !netOutBuffer.hasRemaining();
     }
-    
+
     /**
      * Flushes the buffer to the network, non blocking
      * @param buf ByteBuffer
@@ -127,7 +144,7 @@ public class SecureNioChannel extends NioChannel  {
             return true;
         }
     }
-    
+
     /**
      * Performs SSL handshake, non blocking, but performs NEED_TASK on the same thread.<br>
      * Hence, you should never call this method using your Acceptor thread, as you would slow down
@@ -139,68 +156,141 @@ public class SecureNioChannel extends NioChannel  {
      * @return int - 0 if hand shake is complete, otherwise it returns a SelectionKey interestOps value
      * @throws IOException
      */
+    @Override
     public int handshake(boolean read, boolean write) throws IOException {
-        if ( initHandshakeComplete ) return 0; //we have done our initial handshake
-        
+        if ( handshakeComplete ) return 0; //we have done our initial handshake
+
         if (!flush(netOutBuffer)) return SelectionKey.OP_WRITE; //we still have data to write
-        
+
         SSLEngineResult handshake = null;
-        
-        while (!initHandshakeComplete) {
-            switch ( initHandshakeStatus ) {
+
+        while (!handshakeComplete) {
+            switch ( handshakeStatus ) {
                 case NOT_HANDSHAKING: {
                     //should never happen
                     throw new IOException("NOT_HANDSHAKING during handshake");
                 }
                 case FINISHED: {
                     //we are complete if we have delivered the last package
-                    initHandshakeComplete = !netOutBuffer.hasRemaining();
+                    handshakeComplete = !netOutBuffer.hasRemaining();
                     //return 0 if we are complete, otherwise we still have data to write
-                    return initHandshakeComplete?0:SelectionKey.OP_WRITE; 
+                    return handshakeComplete?0:SelectionKey.OP_WRITE;
                 }
                 case NEED_WRAP: {
                     //perform the wrap function
-                    handshake = handshakeWrap(write);
-                    if ( handshake.getStatus() == Status.OK ){
-                        if (initHandshakeStatus == HandshakeStatus.NEED_TASK) 
-                            initHandshakeStatus = tasks();
+                    try {
+                        handshake = handshakeWrap(write);
+                    } catch (SSLException e) {
+                        if (log.isDebugEnabled()) {
+                            log.debug(sm.getString("channel.nio.ssl.wrapException"), e);
+                        }
+                        handshake = handshakeWrap(write);
+                    }
+                    if (handshake.getStatus() == Status.OK) {
+                        if (handshakeStatus == HandshakeStatus.NEED_TASK)
+                            handshakeStatus = tasks();
+                    } else if (handshake.getStatus() == Status.CLOSED) {
+                        flush(netOutBuffer);
+                        return -1;
                     } else {
                         //wrap should always work with our buffers
                         throw new IOException("Unexpected status:" + handshake.getStatus() + " during handshake WRAP.");
                     }
-                    if ( initHandshakeStatus != HandshakeStatus.NEED_UNWRAP || (!flush(netOutBuffer)) ) {
+                    if ( handshakeStatus != HandshakeStatus.NEED_UNWRAP || (!flush(netOutBuffer)) ) {
                         //should actually return OP_READ if we have NEED_UNWRAP
                         return SelectionKey.OP_WRITE;
                     }
-                    //fall down to NEED_UNWRAP on the same call, will result in a 
+                    //fall down to NEED_UNWRAP on the same call, will result in a
                     //BUFFER_UNDERFLOW if it needs data
                 }
+                //$FALL-THROUGH$
                 case NEED_UNWRAP: {
                     //perform the unwrap function
                     handshake = handshakeUnwrap(read);
                     if ( handshake.getStatus() == Status.OK ) {
-                        if (initHandshakeStatus == HandshakeStatus.NEED_TASK) 
-                            initHandshakeStatus = tasks();
+                        if (handshakeStatus == HandshakeStatus.NEED_TASK)
+                            handshakeStatus = tasks();
                     } else if ( handshake.getStatus() == Status.BUFFER_UNDERFLOW ){
                         //read more data, reregister for OP_READ
                         return SelectionKey.OP_READ;
                     } else {
-                        throw new IOException("Invalid handshake status:"+initHandshakeStatus+" during handshake UNWRAP.");
+                        throw new IOException("Invalid handshake status:"+handshakeStatus+" during handshake UNWRAP.");
                     }//switch
                     break;
                 }
                 case NEED_TASK: {
-                    initHandshakeStatus = tasks();
+                    handshakeStatus = tasks();
                     break;
                 }
-                default: throw new IllegalStateException("Invalid handshake status:"+initHandshakeStatus);
+                default: throw new IllegalStateException("Invalid handshake status:"+handshakeStatus);
             }//switch
-        }//while      
-        //return 0 if we are complete, otherwise reregister for any activity that 
+        }//while
+        //return 0 if we are complete, otherwise reregister for any activity that
         //would cause this method to be called again.
-        return initHandshakeComplete?0:(SelectionKey.OP_WRITE|SelectionKey.OP_READ);
+        return handshakeComplete?0:(SelectionKey.OP_WRITE|SelectionKey.OP_READ);
     }
-    
+
+    /**
+     * Force a blocking handshake to take place for this key.
+     * This requires that both network and application buffers have been emptied out prior to this call taking place, or a
+     * IOException will be thrown.
+     * @param timeout - timeout in milliseconds for each socket operation
+     * @throws IOException - if an IO exception occurs or if application or network buffers contain data
+     * @throws SocketTimeoutException - if a socket operation timed out
+     */
+    public void rehandshake(long timeout) throws IOException {
+        //validate the network buffers are empty
+        if (netInBuffer.position() > 0 && netInBuffer.position()<netInBuffer.limit()) throw new IOException("Network input buffer still contains data. Handshake will fail.");
+        if (netOutBuffer.position() > 0 && netOutBuffer.position()<netOutBuffer.limit()) throw new IOException("Network output buffer still contains data. Handshake will fail.");
+        if (getBufHandler().getReadBuffer().position()>0 && getBufHandler().getReadBuffer().position()<getBufHandler().getReadBuffer().limit()) throw new IOException("Application input buffer still contains data. Data would have been lost.");
+        if (getBufHandler().getWriteBuffer().position()>0 && getBufHandler().getWriteBuffer().position()<getBufHandler().getWriteBuffer().limit()) throw new IOException("Application output buffer still contains data. Data would have been lost.");
+        reset();
+        boolean isReadable = true;
+        boolean isWriteable = true;
+        boolean handshaking = true;
+        Selector selector = null;
+        SelectionKey key = null;
+        try {
+            while (handshaking) {
+                int hsStatus = this.handshake(isReadable, isWriteable);
+                switch (hsStatus) {
+                    case -1 : throw new EOFException("EOF during handshake.");
+                    case  0 : handshaking = false; break;
+                    default : {
+                        long now = System.currentTimeMillis();
+                        if (selector==null) {
+                            synchronized (Selector.class) {
+                                // Selector.open() isn't thread safe
+                                // http://bugs.sun.com/view_bug.do?bug_id=6427854
+                                // Affects 1.6.0_29, fixed in 1.7.0_01
+                                selector = Selector.open();
+                            }
+                            key = getIOChannel().register(selector, hsStatus);
+                        } else {
+                            key.interestOps(hsStatus);
+                        }
+                        int keyCount = selector.select(timeout);
+                        if (keyCount == 0 && ((System.currentTimeMillis()-now) >= timeout)) {
+                            throw new SocketTimeoutException("Handshake operation timed out.");
+                        }
+                        isReadable = key.isReadable();
+                        isWriteable = key.isWritable();
+                    }
+                }
+            }
+        } catch (IOException x) {
+            throw x;
+        } catch (Exception cx) {
+            IOException x = new IOException(cx);
+            throw x;
+        } finally {
+            if (key!=null) try {key.cancel();} catch (Exception ignore) {}
+            if (selector!=null) try {selector.close();} catch (Exception ignore) {}
+        }
+    }
+
+
+
     /**
      * Executes all the tasks needed on the same thread.
      * @return HandshakeStatus
@@ -228,12 +318,12 @@ public class SecureNioChannel extends NioChannel  {
         //prepare the results to be written
         netOutBuffer.flip();
         //set the status
-        initHandshakeStatus = result.getHandshakeStatus();
+        handshakeStatus = result.getHandshakeStatus();
         //optimization, if we do have a writable channel, write it now
         if ( doWrite ) flush(netOutBuffer);
         return result;
     }
-    
+
     /**
      * Perform handshake unwrap
      * @param doread boolean
@@ -241,7 +331,7 @@ public class SecureNioChannel extends NioChannel  {
      * @throws IOException
      */
     protected SSLEngineResult handshakeUnwrap(boolean doread) throws IOException {
-        
+
         if (netInBuffer.position() == netInBuffer.limit()) {
             //clear the buffer if we have emptied it out on data
             netInBuffer.clear();
@@ -250,7 +340,7 @@ public class SecureNioChannel extends NioChannel  {
             //if we have data to read, read it
             int read = sc.read(netInBuffer);
             if (read == -1) throw new IOException("EOF encountered during handshake.");
-        }        
+        }
         SSLEngineResult result;
         boolean cont = false;
         //loop while we can perform pure SSLEngine data
@@ -262,19 +352,19 @@ public class SecureNioChannel extends NioChannel  {
             //compact the buffer, this is an optional method, wonder what would happen if we didn't
             netInBuffer.compact();
             //read in the status
-            initHandshakeStatus = result.getHandshakeStatus();
+            handshakeStatus = result.getHandshakeStatus();
             if ( result.getStatus() == SSLEngineResult.Status.OK &&
                  result.getHandshakeStatus() == HandshakeStatus.NEED_TASK ) {
                 //execute tasks if we need to
-                initHandshakeStatus = tasks();
+                handshakeStatus = tasks();
             }
             //perform another unwrap?
             cont = result.getStatus() == SSLEngineResult.Status.OK &&
-                   initHandshakeStatus == HandshakeStatus.NEED_UNWRAP;
+                   handshakeStatus == HandshakeStatus.NEED_UNWRAP;
         }while ( cont );
         return result;
     }
-    
+
     /**
      * Sends a SSL close message, will not physically close the connection here.<br>
      * To close the connection, you could do something like
@@ -285,7 +375,9 @@ public class SecureNioChannel extends NioChannel  {
      * </code></pre>
      * @throws IOException if an I/O error occurs
      * @throws IOException if there is data on the outgoing network buffer and we are unable to flush it
+     * TODO Implement this java.io.Closeable method
      */
+    @Override
     public void close() throws IOException {
         if (closing) return;
         closing = true;
@@ -316,6 +408,7 @@ public class SecureNioChannel extends NioChannel  {
      * @param force boolean
      * @throws IOException
      */
+    @Override
     public void close(boolean force) throws IOException {
         try {
             close();
@@ -335,20 +428,22 @@ public class SecureNioChannel extends NioChannel  {
      * @return The number of bytes read, possibly zero, or <tt>-1</tt> if the channel has reached end-of-stream
      * @throws IOException If some other I/O error occurs
      * @throws IllegalArgumentException if the destination buffer is different than bufHandler.getReadBuffer()
+     * TODO Implement this java.nio.channels.ReadableByteChannel method
      */
+    @Override
     public int read(ByteBuffer dst) throws IOException {
         //if we want to take advantage of the expand function, make sure we only use the ApplicationBufferHandler's buffers
         if ( dst != bufHandler.getReadBuffer() ) throw new IllegalArgumentException("You can only read using the application read buffer provided by the handler.");
         //are we in the middle of closing or closed?
         if ( closing || closed) return -1;
         //did we finish our handshake?
-        if (!initHandshakeComplete) throw new IllegalStateException("Handshake incomplete, you must complete handshake before reading data.");
+        if (!handshakeComplete) throw new IllegalStateException("Handshake incomplete, you must complete handshake before reading data.");
 
         //read from the network
         int netread = sc.read(netInBuffer);
         //did we reach EOF? if so send EOF up one layer.
         if (netread == -1) return -1;
-        
+
         //the data read
         int read = 0;
         //the SSL engine result
@@ -360,7 +455,7 @@ public class SecureNioChannel extends NioChannel  {
             unwrap = sslEngine.unwrap(netInBuffer, dst);
             //compact the buffer
             netInBuffer.compact();
-            
+
             if ( unwrap.getStatus()==Status.OK || unwrap.getStatus()==Status.BUFFER_UNDERFLOW ) {
                 //we did receive some data, add it to our total
                 read += unwrap.bytesProduced();
@@ -369,7 +464,7 @@ public class SecureNioChannel extends NioChannel  {
                 //if we need more network data, then bail out for now.
                 if ( unwrap.getStatus() == Status.BUFFER_UNDERFLOW ) break;
             }else if ( unwrap.getStatus()==Status.BUFFER_OVERFLOW && read>0 ) {
-                //buffer overflow can happen, if we have read data, then 
+                //buffer overflow can happen, if we have read data, then
                 //empty out the dst buffer before we do another read
                 break;
             }else {
@@ -388,8 +483,11 @@ public class SecureNioChannel extends NioChannel  {
      * @param src The buffer from which bytes are to be retrieved
      * @return The number of bytes written, possibly zero
      * @throws IOException If some other I/O error occurs
+     * TODO Implement this java.nio.channels.WritableByteChannel method
      */
+    @Override
     public int write(ByteBuffer src) throws IOException {
+        checkInterruptStatus();
         if ( src == this.netOutBuffer ) {
             //we can get here through a recursive call
             //by using the NioBlockingSelector
@@ -422,7 +520,7 @@ public class SecureNioChannel extends NioChannel  {
                 if (result.getHandshakeStatus() == HandshakeStatus.NEED_TASK) tasks();
             } else {
                 throw new IOException("Unable to wrap data, invalid engine state: " +result.getStatus());
-            }        
+            }
 
             //force a flush
             flush(netOutBuffer);
@@ -430,12 +528,12 @@ public class SecureNioChannel extends NioChannel  {
             return written;
         }
     }
-    
+
     @Override
     public int getOutboundRemaining() {
         return netOutBuffer.remaining();
     }
-    
+
     @Override
     public boolean flushOutbound() throws IOException {
         int remaining = netOutBuffer.remaining();
@@ -444,7 +542,7 @@ public class SecureNioChannel extends NioChannel  {
         return remaining2 < remaining;
     }
 
-    
+
     /**
      * Callback interface to be able to expand buffers
      * when buffer overflow exceptions happen
@@ -455,14 +553,17 @@ public class SecureNioChannel extends NioChannel  {
         public ByteBuffer getWriteBuffer();
     }
 
+    @Override
     public ApplicationBufferHandler getBufHandler() {
         return bufHandler;
     }
 
-    public boolean isInitHandshakeComplete() {
-        return initHandshakeComplete;
+    @Override
+    public boolean isHandshakeComplete() {
+        return handshakeComplete;
     }
 
+    @Override
     public boolean isClosing() {
         return closing;
     }
@@ -478,9 +579,9 @@ public class SecureNioChannel extends NioChannel  {
     public void setBufHandler(ApplicationBufferHandler bufHandler) {
         this.bufHandler = bufHandler;
     }
-    
+
+    @Override
     public SocketChannel getIOChannel() {
         return sc;
     }
-
-}
\ No newline at end of file
+}
