@@ -23,8 +23,14 @@ import java.io.InputStreamReader;
 import java.io.Reader;
 import java.net.URL;
 import java.net.URLConnection;
+import java.security.Principal;
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
 import java.util.concurrent.atomic.AtomicInteger;
 
+import javax.naming.Context;
 import javax.naming.InitialContext;
 import javax.naming.NamingException;
 import javax.servlet.ServletException;
@@ -41,13 +47,14 @@ import static org.junit.Assert.fail;
 
 import org.junit.Test;
 
-import org.apache.catalina.Context;
 import org.apache.catalina.Host;
 import org.apache.catalina.core.StandardContext;
 import org.apache.catalina.core.StandardHost;
 import org.apache.catalina.deploy.ContextEnvironment;
 import org.apache.catalina.deploy.ContextResourceLink;
 import org.apache.catalina.ha.context.ReplicatedContext;
+import org.apache.catalina.realm.GenericPrincipal;
+import org.apache.catalina.realm.RealmBase;
 import org.apache.tomcat.util.buf.ByteChunk;
 
 public class TestTomcat extends TomcatBaseTest {
@@ -98,14 +105,11 @@ public class TestTomcat extends TomcatBaseTest {
             String name = null;
 
             try {
-                javax.naming.Context initCtx = new InitialContext();
-                javax.naming.Context envCtx =
-                        (javax.naming.Context) initCtx.lookup("java:comp/env");
+                Context initCtx = new InitialContext();
+                Context envCtx = (Context) initCtx.lookup("java:comp/env");
                 name = (String) envCtx.lookup(JNDI_ENV_NAME);
             } catch (NamingException e) {
-                IOException ioe = new IOException(e.getMessage());
-                ioe.initCause(e);
-                throw ioe;
+                throw new IOException(e);
             }
 
             res.getWriter().write("Hello, " + name);
@@ -122,7 +126,7 @@ public class TestTomcat extends TomcatBaseTest {
         @Override
         public void doGet(HttpServletRequest req, HttpServletResponse res)
         throws IOException {
-            URL url = getServletContext().getResource("/WEB-INF/web.xml");
+            URL url = req.getServletContext().getResource("/WEB-INF/web.xml");
 
             res.getWriter().write("The URL obtained for /WEB-INF/web.xml was ");
             if (url == null) {
@@ -190,45 +194,45 @@ public class TestTomcat extends TomcatBaseTest {
     }
 
 
-//    /**
-//     * Simple Realm that uses a configurable {@link Map} to link user names and
-//     * passwords.
-//     */
-//    public static final class MapRealm extends RealmBase {
-//        private Map<String,String> users = new HashMap<String,String>();
-//        private Map<String,List<String>> roles =
-//            new HashMap<String,List<String>>();
-//
-//        public void addUser(String username, String password) {
-//            users.put(username, password);
-//        }
-//
-//        public void addUserRole(String username, String role) {
-//            List<String> userRoles = roles.get(username);
-//            if (userRoles == null) {
-//                userRoles = new ArrayList<String>();
-//                roles.put(username, userRoles);
-//            }
-//            userRoles.add(role);
-//        }
-//
-//        @Override
-//        protected String getName() {
-//            return "MapRealm";
-//        }
-//
-//        @Override
-//        protected String getPassword(String username) {
-//            return users.get(username);
-//        }
-//
-//        @Override
-//        protected Principal getPrincipal(String username) {
-//            return new GenericPrincipal(username, getPassword(username),
-//                    roles.get(username));
-//        }
-//
-//    }
+    /**
+     * Simple Realm that uses a configurable {@link Map} to link user names and
+     * passwords.
+     */
+    public static final class MapRealm extends RealmBase {
+        private Map<String,String> users = new HashMap<String,String>();
+        private Map<String,List<String>> roles =
+            new HashMap<String,List<String>>();
+
+        public void addUser(String username, String password) {
+            users.put(username, password);
+        }
+
+        public void addUserRole(String username, String role) {
+            List<String> userRoles = roles.get(username);
+            if (userRoles == null) {
+                userRoles = new ArrayList<String>();
+                roles.put(username, userRoles);
+            }
+            userRoles.add(role);
+        }
+
+        @Override
+        protected String getName() {
+            return "MapRealm";
+        }
+
+        @Override
+        protected String getPassword(String username) {
+            return users.get(username);
+        }
+
+        @Override
+        protected Principal getPrincipal(String username) {
+            return new GenericPrincipal(username, getPassword(username),
+                    roles.get(username));
+        }
+
+    }
 
     /**
      * Start tomcat with a single context and one
@@ -242,8 +246,7 @@ public class TestTomcat extends TomcatBaseTest {
         Tomcat tomcat = getTomcatInstance();
 
         // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
-
+        org.apache.catalina.Context ctx = tomcat.addContext("", null);
         // You can customize the context by calling
         // its API
 
@@ -261,15 +264,14 @@ public class TestTomcat extends TomcatBaseTest {
         Tomcat tomcat = getTomcatInstance();
 
         File appDir = new File(getBuildDirectory(), "webapps/examples");
-
+        // app dir is relative to server home
         tomcat.addWebapp(null, "/examples", appDir.getAbsolutePath());
 
         tomcat.start();
 
         ByteChunk res = getUrl("http://localhost:" + getPort() +
                 "/examples/servlets/servlet/HelloWorldExample");
-        String text = res.toString();
-        assertTrue(text, text.indexOf("<h1>Hello World!</h1>") > 0);
+        assertTrue(res.toString().indexOf("<h1>Hello World!</h1>") > 0);
     }
 
     @Test
@@ -284,8 +286,7 @@ public class TestTomcat extends TomcatBaseTest {
 
         ByteChunk res = getUrl("http://localhost:" + getPort() +
                 "/examples/jsp/jsp2/el/basic-arithmetic.jsp");
-        String text = res.toString();
-        assertTrue(text, text.indexOf("<td>${(1==2) ? 3 : 4}</td>") > 0);
+        assertTrue(res.toString().indexOf("<td>${(1==2) ? 3 : 4}</td>") > 0);
     }
 
     @Test
@@ -293,7 +294,7 @@ public class TestTomcat extends TomcatBaseTest {
         Tomcat tomcat = getTomcatInstance();
 
         // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
+        org.apache.catalina.Context ctx = tomcat.addContext("", null);
         // You can customize the context by calling
         // its API
 
@@ -325,7 +326,7 @@ public class TestTomcat extends TomcatBaseTest {
         Tomcat tomcat = getTomcatInstance();
 
         // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
+        org.apache.catalina.Context ctx = tomcat.addContext("", null);
 
         // You can customise the context by calling its API
 
@@ -355,7 +356,7 @@ public class TestTomcat extends TomcatBaseTest {
         Tomcat tomcat = getTomcatInstance();
 
         // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
+        org.apache.catalina.Context ctx = tomcat.addContext("", null);
 
         // You can customise the context by calling its API
 
@@ -394,7 +395,7 @@ public class TestTomcat extends TomcatBaseTest {
 
         File appDir = new File(getBuildDirectory(), "webapps" + contextPath);
         // app dir is relative to server home
-        Context ctx =
+        org.apache.catalina.Context ctx =
             tomcat.addWebapp(null, "/examples", appDir.getAbsolutePath());
 
         Tomcat.addServlet(ctx, "testGetResource", new GetResource());
@@ -434,7 +435,7 @@ public class TestTomcat extends TomcatBaseTest {
         Tomcat tomcat = getTomcatInstance();
 
         // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
+        org.apache.catalina.Context ctx = tomcat.addContext("", null);
 
         InitCount initCount = new InitCount();
         Tomcat.addServlet(ctx, "initCount", initCount);
@@ -460,102 +461,100 @@ public class TestTomcat extends TomcatBaseTest {
         assertNull(tomcat.getWebappConfigFile("test/deployment/dirNoContext", ""));
     }
 
-// Reading context.xml from a war file without copying it
-// is not implemented in Tomcat 6. (BZ 48662, r928380)
-//    @Test
-//    public void testGetWebappConfigFileFromJar() {
-//        Tomcat tomcat = new Tomcat();
-//        assertNotNull(tomcat.getWebappConfigFile("test/deployment/context.war", ""));
-//    }
-//
-//    @Test
-//    public void testGetWebappConfigFileFromJarNegative() {
-//        Tomcat tomcat = new Tomcat();
-//        assertNull(tomcat.getWebappConfigFile("test/deployment/noContext.war", ""));
-//    }
-//
-//    @Test
-//    public void testBug51526() throws Exception {
-//        Tomcat tomcat = getTomcatInstance();
-//
-//        File appFile = new File("test/deployment/context.war");
-//        StandardContext context = (StandardContext) tomcat.addWebapp(null, "/test",
-//                appFile.getAbsolutePath());
-//
-//        tomcat.start();
-//
-//        assertEquals("WAR_CONTEXT", context.getSessionCookieName());
-//    }
-//
-//    @Test
-//    public void testGetDefaultContextPerAddWebapp() {
-//        Tomcat tomcat = getTomcatInstance();
-//
-//        File appFile = new File("test/deployment/context.war");
-//        Context context = tomcat.addWebapp(null,
-//                "/test", appFile.getAbsolutePath());
-//
-//        assertEquals(StandardContext.class.getName(), context.getClass()
-//                .getName());
-//    }
-//
-//    @Test
-//    public void testGetBrokenContextPerAddWepapp() {
-//        Tomcat tomcat = getTomcatInstance();
-//        Host host = tomcat.getHost();
-//        if (host instanceof StandardHost) {
-//            ((StandardHost) host).setContextClass("InvalidContextClassName");
-//        }
-//
-//        try {
-//            File appFile = new File("test/deployment/context.war");
-//            tomcat.addWebapp(null, "/test", appFile.getAbsolutePath());
-//            fail();
-//        } catch (IllegalArgumentException e) {
-//            // OK
-//        }
-//    }
-//
-//    @Test
-//    public void testGetCustomContextPerAddWebappWithNullHost() {
-//        Tomcat tomcat = getTomcatInstance();
-//        Host host = tomcat.getHost();
-//        if (host instanceof StandardHost) {
-//            ((StandardHost) host).setContextClass(ReplicatedContext.class
-//                    .getName());
-//        }
-//
-//        File appFile = new File("test/deployment/context.war");
-//        Context context = tomcat.addWebapp(null, "/test",
-//                appFile.getAbsolutePath());
-//
-//        assertEquals(ReplicatedContext.class.getName(), context.getClass()
-//                .getName());
-//    }
-//
-//    @Test
-//    public void testGetCustomContextPerAddWebappWithHost() {
-//        Tomcat tomcat = getTomcatInstance();
-//        Host host = tomcat.getHost();
-//        if (host instanceof StandardHost) {
-//            ((StandardHost) host).setContextClass(ReplicatedContext.class
-//                    .getName());
-//        }
-//
-//        File appFile = new File("test/deployment/context.war");
-//        Context context = tomcat.addWebapp(host, "/test",
-//                appFile.getAbsolutePath());
-//
-//        assertEquals(ReplicatedContext.class.getName(), context.getClass()
-//                .getName());
-//    }
+    @Test
+    public void testGetWebappConfigFileFromJar() {
+        Tomcat tomcat = new Tomcat();
+        assertNotNull(tomcat.getWebappConfigFile("test/deployment/context.war", ""));
+    }
 
     @Test
+    public void testGetWebappConfigFileFromJarNegative() {
+        Tomcat tomcat = new Tomcat();
+        assertNull(tomcat.getWebappConfigFile("test/deployment/noContext.war", ""));
+    }
+
+    @Test
+    public void testBug51526() throws Exception {
+        Tomcat tomcat = getTomcatInstance();
+
+        File appFile = new File("test/deployment/context.war");
+        StandardContext context = (StandardContext) tomcat.addWebapp(null, "/test",
+                appFile.getAbsolutePath());
+
+        tomcat.start();
+
+        assertEquals("WAR_CONTEXT", context.getSessionCookieName());
+    }
+
+    @Test
+    public void testGetDefaultContextPerAddWebapp() {
+        Tomcat tomcat = getTomcatInstance();
+
+        File appFile = new File("test/deployment/context.war");
+        org.apache.catalina.Context context = tomcat.addWebapp(null,
+                "/test", appFile.getAbsolutePath());
+
+        assertEquals(StandardContext.class.getName(), context.getClass()
+                .getName());
+    }
+
+    @Test
+    public void testGetBrokenContextPerAddWepapp() {
+        Tomcat tomcat = getTomcatInstance();
+        Host host = tomcat.getHost();
+        if (host instanceof StandardHost) {
+            ((StandardHost) host).setContextClass("InvalidContextClassName");
+        }
+
+        try {
+            File appFile = new File("test/deployment/context.war");
+            tomcat.addWebapp(null, "/test", appFile.getAbsolutePath());
+            fail();
+        } catch (IllegalArgumentException e) {
+            // OK
+        }
+    }
+
+    @Test
+    public void testGetCustomContextPerAddWebappWithNullHost() {
+        Tomcat tomcat = getTomcatInstance();
+        Host host = tomcat.getHost();
+        if (host instanceof StandardHost) {
+            ((StandardHost) host).setContextClass(ReplicatedContext.class
+                    .getName());
+        }
+
+        File appFile = new File("test/deployment/context.war");
+        org.apache.catalina.Context context = tomcat.addWebapp(null, "/test",
+                appFile.getAbsolutePath());
+
+        assertEquals(ReplicatedContext.class.getName(), context.getClass()
+                .getName());
+    }
+
+    @Test
+    public void testGetCustomContextPerAddWebappWithHost() {
+        Tomcat tomcat = getTomcatInstance();
+        Host host = tomcat.getHost();
+        if (host instanceof StandardHost) {
+            ((StandardHost) host).setContextClass(ReplicatedContext.class
+                    .getName());
+        }
+
+        File appFile = new File("test/deployment/context.war");
+        org.apache.catalina.Context context = tomcat.addWebapp(host, "/test",
+                appFile.getAbsolutePath());
+
+        assertEquals(ReplicatedContext.class.getName(), context.getClass()
+                .getName());
+    }
+
+        @Test
     public void testGetDefaultContextPerAddContext() {
         Tomcat tomcat = getTomcatInstance();
 
         // No file system docBase required
-        Context ctx = tomcat.addContext(null, "", null);
+        org.apache.catalina.Context ctx = tomcat.addContext(null, "", null);
         assertEquals(StandardContext.class.getName(), ctx.getClass().getName());
     }
 
@@ -586,7 +585,7 @@ public class TestTomcat extends TomcatBaseTest {
         }
 
         // No file system docBase required
-        Context ctx = tomcat.addContext(host, "", null);
+        org.apache.catalina.Context ctx = tomcat.addContext(host, "", null);
         assertEquals(ReplicatedContext.class.getName(), ctx.getClass()
                 .getName());
     }
