@@ -20,10 +20,7 @@ import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
-import java.net.InetAddress;
-import java.net.Socket;
 import java.net.URISyntaxException;
-import java.net.UnknownHostException;
 import java.security.KeyManagementException;
 import java.security.KeyStore;
 import java.security.NoSuchAlgorithmException;
@@ -33,8 +30,6 @@ import javax.net.ssl.KeyManager;
 import javax.net.ssl.KeyManagerFactory;
 import javax.net.ssl.SSLContext;
 import javax.net.ssl.SSLServerSocketFactory;
-import javax.net.ssl.SSLSocket;
-import javax.net.ssl.SSLSocketFactory;
 import javax.net.ssl.TrustManager;
 import javax.net.ssl.TrustManagerFactory;
 import javax.net.ssl.X509TrustManager;
@@ -46,11 +41,13 @@ import javax.servlet.http.HttpServletResponse;
 import org.apache.catalina.Context;
 import org.apache.catalina.authenticator.SSLAuthenticator;
 import org.apache.catalina.connector.Connector;
+import org.apache.catalina.core.AprLifecycleListener;
 import org.apache.catalina.deploy.LoginConfig;
 import org.apache.catalina.deploy.SecurityCollection;
 import org.apache.catalina.deploy.SecurityConstraint;
 import org.apache.catalina.startup.TestTomcat.MapRealm;
 import org.apache.catalina.startup.Tomcat;
+import org.apache.tomcat.jni.SSL;
 
 public final class TesterSupport {
 
@@ -127,6 +124,16 @@ public final class TesterSupport {
         }
         tomcat.getConnector().setSecure(true);
         tomcat.getConnector().setProperty("SSLEnabled", "true");
+        // OpenSSL before 1.0.1 only supports TLSv1.
+        // Our default SSLProtocol setting "all" includes unsupported TLSv1.1 and 1.2
+        // and would produce an error during init.
+        // Furthermore old Java 6 uses SSLv2Hello which OpenSSL only
+        // supports if we choose multiple protocols.
+        // Trigger loading of the native library and choose old protocol
+        // if we use old OpenSSL.
+        if (AprLifecycleListener.isAprAvailable() && SSL.version() < 0x10001000L) {
+            tomcat.getConnector().setProperty("SSLProtocol", Constants.SSL_PROTO_TLSv1 + "+" + Constants.SSL_PROTO_SSLv3);
+        }
     }
 
     private static File toFile(java.net.URL url) {
@@ -155,12 +162,11 @@ public final class TesterSupport {
 
     protected static void configureClientSsl() {
         try {
-            SSLContext sc = SSLContext.getInstance("SSL");
+            SSLContext sc = SSLContext.getInstance("TLS");
             sc.init(TesterSupport.getUser1KeyManagers(),
                     TesterSupport.getTrustManagers(),
                     null);
-            javax.net.ssl.HttpsURLConnection.setDefaultSSLSocketFactory(
-                    new TesterSSLSocketFactory(sc.getSocketFactory()));
+            javax.net.ssl.HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
         } catch (Exception e) {
             e.printStackTrace();
         }
@@ -200,9 +206,8 @@ public final class TesterSupport {
         TesterSupport.initSsl(tomcat);
 
         // Need a web application with a protected and unprotected URL
-        // Must have a real docBase - just use temp
-        Context ctx =
-            tomcat.addContext("", System.getProperty("java.io.tmpdir"));
+        // No file system docBase required
+        Context ctx = tomcat.addContext("", null);
 
         Tomcat.addServlet(ctx, "simple", new SimpleServlet());
         ctx.addServletMapping("/unprotected", "simple");
@@ -292,67 +297,4 @@ public final class TesterSupport {
             // NOOP - Trust everything
         }
     }
-
-    private static class TesterSSLSocketFactory
-            extends SSLSocketFactory {
-
-        private SSLSocketFactory factory;
-
-        public TesterSSLSocketFactory(SSLSocketFactory factory) {
-            this.factory = factory;
-        }
-
-        @Override
-        public String[] getDefaultCipherSuites() {
-            return factory.getDefaultCipherSuites();
-        }
-
-        @Override
-        public String[] getSupportedCipherSuites() {
-            return factory.getSupportedCipherSuites();
-        }
-
-        @Override
-        public Socket createSocket(Socket socket, String s, int i, boolean flag)
-                throws IOException {
-            SSLSocket result =
-                (SSLSocket) factory.createSocket(socket, s, i, flag);
-            result.setEnabledProtocols(new String[] { "SSLv3" } );
-            return result;
-        }
-
-        @Override
-        public Socket createSocket(String s, int i) throws IOException,
-                UnknownHostException {
-            SSLSocket result = (SSLSocket) factory.createSocket(s, i);
-            result.setEnabledProtocols(new String[] { "SSLv3" } );
-            return result;
-        }
-
-        @Override
-        public Socket createSocket(String s, int i, InetAddress inetaddress,
-                int j) throws IOException, UnknownHostException {
-            SSLSocket result =
-                (SSLSocket) factory.createSocket(s, i, inetaddress, j);
-            result.setEnabledProtocols(new String[] { "SSLv3" } );
-            return result;
-        }
-
-        @Override
-        public Socket createSocket(InetAddress inetaddress, int i)
-                throws IOException {
-            SSLSocket result = (SSLSocket) factory.createSocket(inetaddress, i);
-            result.setEnabledProtocols(new String[] { "SSLv3" } );
-            return result;
-        }
-
-        @Override
-        public Socket createSocket(InetAddress inetaddress, int i,
-                InetAddress inetaddress1, int j) throws IOException {
-            SSLSocket result = (SSLSocket) factory.createSocket(
-                    inetaddress, i, inetaddress1, j);
-            result.setEnabledProtocols(new String[] { "SSLv3" } );
-            return result;
-        }
-    }
 }
